unit IAChartBaseAxisUnit;

interface
uses Windows, Messages,Controls,SysUtils, Classes,Graphics,Dialogs,Math,IAChartBaseAxisEditUnit;
const
  CM_NEWFONT = CM_BASE+111;
  CM_ORIENTATIONCHANGED =CM_BASE + 112;
  CM_AXISSIZED = CM_BASE + 113;
  CM_AXISMOVED = CM_BASE + 114;
  CM_AXISCLICKED = CM_BASE + 115;
  CM_AXISRECALIBRATED =CM_BASE +116;
  CM_AXISDBLCLICKED = CM_BASE + 117;
  RecommendedSteps : array  [0..8] of double = (0.04,0.05,0.1,0.2,0.4,0.5,1.0,2.0,4.0);


type
IAChartBaseAxisType = (iacbatHorizontal,iacbatVertical);
IAChartBaseAxisStyle =(iacbasLinear);
IAChartBaseAxisClass = class;
IAChartBaseAxisClassData = class(TPersistent)//wrapper for axe
  private
    FAxisField : IAChartBaseAxisClass;
    function GetHeight: integer;
    function GetWidth: integer;
    procedure SetHeight(Value: integer);
    procedure SetWidth(Value: integer);
    function GetLeft: integer;
    function GetTop: integer;
    procedure SetLeft(Value: integer);
    procedure SetTop(Value: integer);
    function GetAxisType: IAChartBaseAxisType;
    function GetBeginShift: Integer;
    function GetCenterPointLeft: integer;
    function GetCenterPointTop: Integer;
    function GetEndShift: Integer;
    procedure SetAxisType(Value: IAChartBaseAxisType);
    procedure SetBeginShift(Value: Integer);
    procedure SetCenterPointLeft(Value: integer);
    procedure SetCenterPointTop(Value: Integer);
    procedure SetEndShift(Value: Integer);
    function GetFont: TFont;
    procedure SetFont(Value: TFont);
    function GetScaleStyle: IAChartBaseAxisStyle;
    procedure SetScaleStyle(Value: IAChartBaseAxisStyle);
    function GetEndValue: Double;
    function GetMinorTicks: byte;
    function GetWantedTicks: byte;
    procedure SetEndValue(Value: Double);
    procedure SetMinorTicks(Value: byte);
    procedure SetWantedTicks(Value: byte);
    function GetBeginValue: Double;
    procedure SetBeginValue(Value: Double);
    function GetCurrImageList: TImageList;
    procedure SetCurrImageList(Value: TImageList);
    function GetColor: TColor;
    procedure SetColor(Value: TColor);
    function GetVisible: Boolean;
    procedure SetVisible(Value: Boolean);
    function GetAutoEditable: Boolean;
    procedure SetAutoEditable(Value: Boolean);

  protected

  public
    function IsYours(AObject :IAChartBaseAxisClass): boolean;
    procedure Assign(Source: TPersistent); override;
    constructor Create(AOwner: TComponent); virtual;
    destructor Destroy; override;
    procedure SetLength(AValue : integer);
    function GetCurrStep: Double;
    function GetRealBeginValue: Double;
    function GetRealEndvalue : Double;
  published
    property AxisType : IAChartBaseAxisType read GetAxisType write SetAxisType;
    property Width : integer read GetWidth write SetWidth;
    property Height : integer read GetHeight write SetHeight;
    property Top : integer read GetTop write SetTop;
    property Left : integer read GetLeft write SetLeft;
    property CenterPointLeft : integer read GetCenterPointLeft write SetCenterPointLeft;
    property CenterPointTop : Integer read GetCenterPointTop write SetCenterPointTop;
    property BeginShift : Integer read GetBeginShift write SetBeginShift;
    property EndShift : Integer read GetEndShift write SetEndShift;

    property Font : TFont read GetFont write SetFont;
    property ScaleStyle : IAChartBaseAxisStyle read GetScaleStyle write SetScaleStyle;
    property BeginValue : Double read GetBeginValue write SetBeginValue;
    property EndValue : Double read GetEndValue write SetEndValue;
    property WantedTicks : byte read GetWantedTicks write SetWantedTicks;
    property MinorTicks : byte read GetMinorTicks write SetMinorTicks;
    property CurrImageList : TImageList read GetCurrImageList write SetCurrImageList;
    property Color : TColor read GetColor write SetColor;
    property Visible : Boolean read GetVisible write SetVisible;
    property AutoEditable : Boolean read GetAutoEditable write SetAutoEditable;
  end;

IAChartBaseAxisClass = class(TGraphicControl)
  private
    IsCreated : boolean;
    FCenterPointLeft: integer;
    FBeginShift: Integer;
    FCenterPointTop: Integer;
    FEndShift: Integer;
    FAxisType: IAChartBaseAxisType;
    FScaleStyle: IAChartBaseAxisStyle;
    FMinorTicks: byte;
    FWantedTicks: byte;
    FEndValue: Double;
    FBeginValue: Double;
    FRealBeginValue,FRealEndValue :Double;

    RealStep : integer;
    RealPower : integer;
    RealShifting : Double;
    FCurrImageList: TImageList;
    FAutoEditable: Boolean;

    procedure WMLButtonDown(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMLButtonDblClk(var Message: TWMLButtonDblClk); message WM_LBUTTONDBLCLK;
    procedure WMMouseMove(var Message: TWMMouseMove); message WM_MOUSEMOVE;
    procedure WMLButtonUp(var Message: TWMLButtonUp); message WM_LBUTTONUP;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;

    function GetHeight: integer;
    function GetWidth: integer;
    procedure SetHeight(Value: integer);
    procedure SetWidth(Value: integer);
    function GetLeft: integer;
    function GetTop: integer;
    procedure SetLeft(Value: integer);
    procedure SetTop(Value: integer);
    procedure SetBeginShift(Value: Integer);
    procedure SetCenterPointLeft(Value: integer);
    procedure SetCenterPointTop(Value: Integer);
    procedure SetEndShift(Value: Integer);
    function CorrectlySizes(ABeginShift,AEndShift : Integer): Boolean;
    procedure SetAxisType(Value: IAChartBaseAxisType);
    procedure IfFontChanged(Sender: TObject);
    function GetFont: TFont;
    procedure SetFont(Value: TFont);
    procedure SetScaleStyle(Value: IAChartBaseAxisStyle);
    procedure Recalibrate;
    procedure SetBeginValue(Value: Double);
    procedure SetEndValue(Value: Double);
    procedure SetMinorTicks(Value: byte);
    procedure SetWantedTicks(Value: byte);
    function UnderlayInt(AValue : Double): integer;
    function UpperlayInt(Avalue : Double): integer;
    function GetRealPower(AValue : Double): integer;
    procedure SetCurrImageList(Value: TImageList);

    procedure DrawMainLine;
    procedure DrawMainTickAt(x,y : integer);
    
    procedure DrawMinorTickAt(x,y : integer);
    procedure DrawMainInfoString;
    function GetColor: TColor;
    procedure SetColor(Value: TColor);
    procedure RotateFont(AFont: TFont; ADeg: Integer);
    function GetVisible: Boolean;
    procedure SetVisible(Value: Boolean);
    procedure SetAutoEditable(Value: Boolean);

  protected
    procedure Paint; override;
  public
    procedure Assign(Source: TPersistent); override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;

    property Canvas;

    property AxisType : IAChartBaseAxisType read FAxisType write SetAxisType;
    property Width : integer read GetWidth write SetWidth;
    property Height : integer read GetHeight write SetHeight;
    property Top : integer read GetTop write SetTop;
    property Left : integer read GetLeft write SetLeft;
    property CenterPointLeft : integer read FCenterPointLeft write SetCenterPointLeft;
    property CenterPointTop : Integer read FCenterPointTop write SetCenterPointTop;
    property BeginShift : Integer read FBeginShift write SetBeginShift;
    property EndShift : Integer read FEndShift write SetEndShift;

    property Font : TFont read GetFont write SetFont;
    property ScaleStyle : IAChartBaseAxisStyle read FScaleStyle write SetScaleStyle;
    property BeginValue : Double read FBeginValue write SetBeginValue;
    property EndValue : Double read FEndValue write SetEndValue;
    property WantedTicks : byte read FWantedTicks write SetWantedTicks;
    property MinorTicks : byte read FMinorTicks write SetMinorTicks;
    property CurrImageList : TImageList read FCurrImageList write SetCurrImageList;
    property Color : TColor read GetColor write SetColor;
    property Visible : Boolean read GetVisible write SetVisible;
    property AutoEditable : Boolean read FAutoEditable write SetAutoEditable;
end;
implementation

{ IAChartBaseAxisClass }

procedure IAChartBaseAxisClass.Assign(Source: TPersistent);
begin
  if Source is IAChartBaseAxisClass then
    begin

      AxisType:=(Source as IAChartBaseAxisClass).AxisType;
      inherited width:=(Source as IAChartBaseAxisClass).width;
      inherited height:=(Source as IAChartBaseAxisClass).height;

      CenterPointLeft:=(Source as IAChartBaseAxisClass).CenterPointLeft;
      BeginShift:=(Source as IAChartBaseAxisClass).BeginShift;
      CenterPointTop:=(Source as IAChartBaseAxisClass).CenterPointTop;
      EndShift:=(Source as IAChartBaseAxisClass).EndShift;

      ScaleStyle:=(Source as IAChartBaseAxisClass).ScaleStyle;
      MinorTicks:=(Source as IAChartBaseAxisClass).MinorTicks;
      WantedTicks:=(Source as IAChartBaseAxisClass).WantedTicks;
      EndValue:=(Source as IAChartBaseAxisClass).EndValue;
      BeginValue:=(Source as IAChartBaseAxisClass).BeginValue;
      CurrImageList:=(Source as IAChartBaseAxisClass).CurrImageList;
      AutoEditable:=(Source as IAChartBaseAxisClass).AutoEditable;
      Font:=(Source as IAChartBaseAxisClass).Font;
      (self as TGraphicControl).Assign((Source as TGraphicControl));
    end else
    begin
      inherited Assign(Source);
    end;
end;

procedure IAChartBaseAxisClass.CMMouseEnter(var Message: TMessage);
begin
  inherited;
end;

procedure IAChartBaseAxisClass.CMMouseLeave(var Message: TMessage);
begin
  inherited;
end;

function IAChartBaseAxisClass.CorrectlySizes(ABeginShift,AEndShift : Integer): Boolean;
begin
  result:=False;
  case FAxisType of
    iacbatHorizontal :
      begin
        if Width>(ABeginShift+AEndShift) then Result:=True;
      end;
    iacbatVertical :
      begin
        if Height>(ABeginShift+AEndShift) then Result:=True;
      end;
  end;
end;

constructor IAChartBaseAxisClass.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  IsCreated:=false;
  Inherited Width:=100;
  inherited height:=32;
  inherited Top:=0;
  Inherited left:=0;
  inherited Font.name:='Times New Roman';
  inherited Font.OnChange:=IfFontChanged;
  inherited Color:=clBlack;
  inherited Visible:=True;
  FAutoEditable:=true;
  FCenterPointLeft:=0;
  FBeginShift:=20;
  FCenterPointTop:=10;
  FEndShift:=50;
  FAxisType:=iacbatHorizontal;
  FScaleStyle:=iacbasLinear;
  FMinorTicks:=1;
  FWantedTicks:=8;
  FEndValue:=1.0;
  FBeginValue:=-1.0;
  FCurrImageList:=nil;

  recalibrate;

  IsCreated:=true;
end;

destructor IAChartBaseAxisClass.Destroy;
begin
  inherited destroy;
end;

procedure IAChartBaseAxisClass.DrawMainInfoString;
var
 s : string;
 Asize: TSize;
 Allsize : TSize;
 Ap : integer;
 Ad : double;
 dx : integer;
begin
 s:='10';ap:=0;ad:=0;
 if RealShifting<>0 then
   begin
     if RealShifting>0 then s:=s+'   + ' else s:=s+'   ';
     ap:=UnderlayInt(log10(abs(RealShifting)));
     ad:=RealShifting/power(10,ap);
     s:=s+format('%7.5f',[ad])+'*10';
   end;
 GetTextExtentPoint32(canvas.handle,pChar(s),length(s),Allsize);
 canvas.Font.Size:=canvas.Font.Size-2;
 if FAxisType =iacbatVertical then
    begin
      RotateFont(canvas.Font,90);
    end;
 if ad<>0 then s:=inttostr(RealPower)+inttostr(ap) else s:=inttostr(RealPower);
 GetTextExtentPoint32(canvas.handle,pChar(s),length(s),Asize);
 Allsize.cx:=Allsize.cx+Asize.cx;

 canvas.Font.Size:=canvas.Font.Size+2;
 if FAxisType =iacbatVertical then
    begin
      RotateFont(canvas.Font,90);
    end;
 //now draw...
 s:='10';dx:=0;
 GetTextExtentPoint32(canvas.handle,pChar(s),length(s),Asize);

 case AxisType of
   iacbatHorizontal :
     begin
       if self.CenterPointTop>(parent.ClientHeight div 2) then
         begin
           canvas.TextOut(width-AllSize.cx,2,s);
         end else
         begin
           canvas.TextOut(width-AllSize.cx,height-AllSize.cy,s);
         end;
     end;
   iacbatVertical :
     begin
       if self.CenterPointLeft>(parent.ClientWidth div 2) then
         begin
           canvas.TextOut(2,AllSize.cx,s);
         end else
         begin
           canvas.TextOut(width-AllSize.cy,AllSize.cx,s);
         end;
     end;
  end;
 dx:=dx+ Asize.cx;
 canvas.Font.Size:=canvas.Font.Size-2;
 if FAxisType =iacbatVertical then
    begin
      RotateFont(canvas.Font,90);
    end;
 s:=inttostr(RealPower);
 GetTextExtentPoint32(canvas.handle,pChar(s),length(s),Asize);
 case AxisType of
   iacbatHorizontal :
     begin
       if self.CenterPointTop>(parent.ClientHeight div 2) then
         begin
           canvas.TextOut(width-AllSize.cx+dx,0,s);
         end else
         begin
           canvas.TextOut(width-AllSize.cx+dx,height-AllSize.cy-2,s);
         end;
     end;
   iacbatVertical :
     begin
       if self.CenterPointLeft>(parent.ClientWidth div 2) then
         begin
           canvas.TextOut(0,AllSize.cx-dx,s);
         end else
         begin
           canvas.TextOut(width-AllSize.cy-2,AllSize.cx-dx,s);
         end;
     end;
  end;
 dx:=dx+ Asize.cx;
 canvas.Font.Size:=canvas.Font.Size+2;
 if FAxisType =iacbatVertical then
    begin
      RotateFont(canvas.Font,90);
    end;
 if RealShifting<>0 then
   begin

     if RealShifting>0 then s:='   + ' else s:='   ';
     ap:=UnderlayInt(log10(abs(RealShifting)));
     ad:=RealShifting/power(10,ap);
     s:=s+ format('%7.5f',[ad])+'*10';
     GetTextExtentPoint32(canvas.handle,pChar(s),length(s),Asize);
     case AxisType of
       iacbatHorizontal :
         begin
           if self.CenterPointTop>(parent.ClientHeight div 2) then
             begin
               canvas.TextOut(width-AllSize.cx+dx,2,s);
             end else
             begin
               canvas.TextOut(width-AllSize.cx+dx,height-AllSize.cy,s);
             end;
         end;
       iacbatVertical :
         begin
           if self.CenterPointLeft>(parent.ClientWidth div 2) then
             begin
               canvas.TextOut(2,AllSize.cx-dx,s);
             end else
             begin
               canvas.TextOut(width-AllSize.cy,AllSize.cx-dx,s);
             end;
         end;
      end;
     dx:=dx+ Asize.cx;
     canvas.Font.Size:=canvas.Font.Size-2;
     if FAxisType =iacbatVertical then
        begin
          RotateFont(canvas.Font,90);
        end;
     s:=inttostr(ap);
     GetTextExtentPoint32(canvas.handle,pChar(s),length(s),Asize);
     case AxisType of
       iacbatHorizontal :
         begin
           if self.CenterPointTop>(parent.ClientHeight div 2) then
             begin
               canvas.TextOut(width-AllSize.cx+dx,0,s);
             end else
             begin
               canvas.TextOut(width-AllSize.cx+dx,height-AllSize.cy-2,s);
             end;
         end;
       iacbatVertical :
         begin
           if self.CenterPointLeft>(parent.ClientWidth div 2) then
             begin
               canvas.TextOut(0,AllSize.cx-dx,s);
             end else
             begin
               canvas.TextOut(width-AllSize.cy-2,AllSize.cx-dx,s);
             end;
         end;
      end;

     canvas.Font.Size:=canvas.Font.Size+2;
     if FAxisType =iacbatVertical then
        begin
          RotateFont(canvas.Font,90);
        end;
  end;
end;

procedure IAChartBaseAxisClass.DrawMainLine;
var
  A_size : integer;
begin
  case AxisType of
    iacbatHorizontal :
      begin
        A_Size :=min(abs(font.size),height div 2);
        canvas.Pen.color:=Color;
        canvas.moveto(0,height div 2);
        canvas.lineto(width,height div 2);
        if (FCurrImageList<>nil) and (assigned(FCurrImageList)) and (FCurrImageList.Count>=3)then
          begin
            FCurrImageList.Draw(canvas,width-FCurrImageList.width,
                                height div 2-FCurrImageList.height div 2,2,True);

          end else
          begin
            canvas.moveto(width,height div 2);
            canvas.lineto(width-A_Size,height div 2-A_Size);
            canvas.lineto(Width-A_Size div 2,height div 2);
            canvas.lineto(width-A_Size,height div 2+A_Size);
            canvas.lineto(width,height div 2);
          end;

      end;
    iacbatVertical   :
      begin
        A_Size :=min(abs(font.size),Width div 2);
        canvas.Pen.color:=Color;
        canvas.moveto(width div 2,0);
        canvas.lineto(width div 2,Height);
        if (FCurrImageList<>nil) and (assigned(FCurrImageList)) and (FCurrImageList.Count>=3)then
          begin
            FCurrImageList.Draw(canvas,width div 2-FCurrImageList.width div 2,0,2,True);

          end else
          begin
            canvas.moveto(width div 2,0);
            canvas.lineto(width div 2-A_size,A_size);
            canvas.lineto(width div 2,A_size div 2);
            canvas.lineto(width div 2+A_size,A_size);
            canvas.lineto(width div 2,0);
          end;
      end;
  end;
end;

procedure IAChartBaseAxisClass.DrawMainTickAt(x, y: integer);
var
  A_size : integer;
begin
  case AxisType of
    iacbatHorizontal :
      begin
        A_Size :=min(abs(font.size),height div 2);
        canvas.Pen.color:=Color;
        if (x>=FBeginShift) and (x<=(width-FEndShift)) then
          begin
            if (FCurrImageList<>nil) and (assigned(FCurrImageList)) and (FCurrImageList.Count>=1)then
              begin
                FCurrImageList.Draw(canvas,x-FCurrImageList.width div 2,y-FCurrImageList.height div 2,0,True);
              end else
              begin
                if self.CenterPointTop>(parent.ClientHeight div 2) then
                  begin
                    canvas.moveto(x,y-A_Size div 2);
                    canvas.lineto(x,y+A_size);
                  end else
                  begin
                    canvas.moveto(x,y-A_Size);
                    canvas.lineto(x,y+A_size div 2);
                  end;
              end;
          end;
      end;
    iacbatVertical   :
      begin
        A_Size :=min(abs(font.size),width div 2);
        canvas.Pen.color:=Color;
        if (y<=(height-FBeginShift)) and (y>=FEndShift) then
          begin
            if (FCurrImageList<>nil) and (assigned(FCurrImageList)) and (FCurrImageList.Count>=1)then
              begin
                FCurrImageList.Draw(canvas,x-FCurrImageList.width div 2,y-FCurrImageList.height div 2,0,True);
              end else
              begin
                if self.CenterPointLeft<(parent.ClientWidth div 2) then
                  begin
                    canvas.moveto(x+A_Size div 2,y);
                    canvas.lineto(x-A_size,y);
                  end else
                  begin
                    canvas.moveto(x+A_Size,y);
                    canvas.lineto(x-A_size div 2,y);
                  end;
              end;
          end;
      end;
  end;  
end;

procedure IAChartBaseAxisClass.DrawMinorTickAt(x, y: integer);
var
  A_size : integer;
begin
  case AxisType of
    iacbatHorizontal :
      begin
        A_Size :=min(abs(font.size),height div 2);
        canvas.Pen.color:=Color;
        if (x>=FBeginShift) and (x<=(width-FEndShift)) then
          begin
            if (FCurrImageList<>nil) and (assigned(FCurrImageList)) and (FCurrImageList.Count>=2)then
              begin
                FCurrImageList.Draw(canvas,x-FCurrImageList.width div 2,y-FCurrImageList.height div 2,1,True);
              end else
              begin
                if self.CenterPointTop>(parent.ClientHeight div 2) then
                  begin
                    canvas.moveto(x,y);
                    canvas.lineto(x,y+A_size);
                  end else
                  begin
                    canvas.moveto(x,y);
                    canvas.lineto(x,y-A_size);
                  end;
              end;
          end;
      end;
    iacbatVertical   :
      begin
        A_Size :=min(abs(font.size),width div 2);
        canvas.Pen.color:=Color;
        if (y<=(height-FBeginShift)) and (y>=FEndShift) then
          begin
            if (FCurrImageList<>nil) and (assigned(FCurrImageList)) and (FCurrImageList.Count>=2)then
              begin
                FCurrImageList.Draw(canvas,x-FCurrImageList.width div 2,y-FCurrImageList.height div 2,1,True);
              end else
              begin
                if self.CenterPointLeft<(parent.ClientWidth div 2) then
                  begin
                    canvas.moveto(x,y);
                    canvas.lineto(x-A_size,y);
                  end else
                  begin
                    canvas.moveto(x,y);
                    canvas.lineto(x+A_size,y);
                  end;
              end;
          end;
      end;
  end;  
end;

function IAChartBaseAxisClass.GetColor: TColor;
begin
  result:=inherited Color;
end;

function IAChartBaseAxisClass.GetFont: TFont;
begin
  result:= inherited Font;
end;

function IAChartBaseAxisClass.GetHeight: integer;
begin
  result:=inherited Height;
end;

function IAChartBaseAxisClass.GetLeft: integer;
begin
  result:=inherited Left;
end;

function IAChartBaseAxisClass.GetRealPower(AValue: Double): integer;
begin
  if AValue<>0 then
    begin
      result:=UnderlayInt(log10(abs(AValue)));
    end else
    begin
      result:=0;
    end;
end;

function IAChartBaseAxisClass.GetTop: integer;
begin
  result:=Inherited Top;
end;

function IAChartBaseAxisClass.GetWidth: integer;
begin
  result:=inherited Width;
end;

procedure IAChartBaseAxisClass.IfFontChanged(Sender: TObject);
begin
  if IsCreated then
    begin
      if FAxisType =iacbatVertical then
        begin
          width:=max(width,abs(4*Font.size));
        end else
        begin
          height:=max(height,abs(4*Font.size));
        end;
      if assigned(self.parent) and not (csCreating  in Parent.ControlState)
        then
          begin
            postmessage(parent.handle,CM_NEWFONT,cardinal(self),0);
            invalidate;
          end;
    end; 
end;

procedure IAChartBaseAxisClass.Paint;
var ABegin,step,AEnd,Avalue,scale : double;
    x,y,i,dd,dh : integer;
    s : string;
begin
  if not (inherited Visible) then exit;
  canvas.Font.Assign(self.font);
  if FAxisType =iacbatVertical then
    begin
      RotateFont(canvas.Font,90);
    end;
  canvas.brush.Style:=bsClear;
  case FScaleStyle of
    iacbasLinear :
      begin
        DrawMainLine;
        case FAxisType of
          iacbatHorizontal :
            begin
              if self.CenterPointTop>(parent.ClientHeight div 2) then dh:=0 else dh:=-2*abs(font.size);
              if FRealBeginValue<FRealEndValue then
               begin
                 step:=power(10,Realpower)*RecommendedSteps[RealStep];
                 ABegin:=UnderlayInt((FRealBeginValue-RealShifting)/step)*step;
                 AEnd:=UpperlayInt((FRealEndValue-RealShifting)/step)*step;
                 AValue:=Abegin;
                 scale:=(width-BeginShift-EndShift)/(FRealEndValue-FRealBeginValue);
                 y:=height div 2;

                 while AValue<=AEnd do
                   begin
                     x:=round(BeginShift+scale*(AValue-FRealBeginValue+RealShifting));
                     DrawMainTickAt(x,y);

                     for i:=1 to FMinorTicks do
                       begin
                         dd:=round(scale*step*i/(FMinorTicks+1));
                         DrawMinorTickAt(x+dd,y);
                       end;
                     s:=format('%5.2f',[AValue/power(10,Realpower)]);
                     if (x>=FBeginShift) and (x<=(width-FEndShift)) then

                     canvas.TextOut(x,y+dh,s);
                     AValue:=AValue+step;
                   end;
                 DrawMainInfoString;
               end else
               begin
                 step:=power(10,Realpower)*RecommendedSteps[RealStep];
                 ABegin:=UnderlayInt((FRealEndValue-RealShifting)/step)*step;
                 AEnd:=UpperlayInt((FRealBeginValue-RealShifting)/step)*step;
                 AValue:=Abegin;
                 scale:=(width-BeginShift-EndShift)/(-FRealEndValue+FRealBeginValue);
                 y:=height div 2;
                 while AValue<=AEnd do
                   begin
                     x:=round(width-1-endShift-scale*(AValue-FRealEndValue+RealShifting));
                     DrawMainTickAt(x,y);

                     for i:=1 to FMinorTicks do
                       begin
                         dd:=round(scale*step*i/(FMinorTicks+1));
                         DrawMinorTickAt(x-dd,y);
                       end;
                     s:=format('%5.2f',[AValue/power(10,Realpower)]);
                     if (x>=FBeginShift) and (x<=(width-FEndShift)) then
                     canvas.TextOut(x,y+dh,s);
                     AValue:=AValue+step;
                   end;
               end;
              DrawMainInfoString;
            end;
          iacbatVertical :
            begin
              if self.CenterPointLeft>(parent.ClientWidth div 2) then dh:=0 else dh:=-2*abs(font.size);
              if FRealBeginValue<FRealEndValue then
               begin
                 step:=power(10,Realpower)*RecommendedSteps[RealStep];
                 ABegin:=UnderlayInt((FRealBeginValue-RealShifting)/step)*step;
                 AEnd:=UpperlayInt((FRealEndValue-RealShifting)/step)*step;
                 AValue:=Abegin;
                 scale:=(height-BeginShift-EndShift)/(FRealEndValue-FRealBeginValue);
                 x:=width div 2;
                 while AValue<=AEnd do
                   begin
                     y:=Height-1-round(BeginShift+scale*(AValue-FRealBeginValue+RealShifting));
                     DrawMainTickAt(x,y);

                     for i:=1 to FMinorTicks do
                       begin
                         dd:=round(scale*step*i/(FMinorTicks+1));
                         DrawMinorTickAt(x,y-dd);
                       end;
                     s:=format('%5.2f',[AValue/power(10,Realpower)]);
                     if (y<=(height-FBeginShift)) and (y>=FEndShift) then
                     canvas.TextOut(x+dh,y,s);
                     AValue:=AValue+step;
                   end;
                 DrawMainInfoString;
               end else
               begin
                 step:=power(10,Realpower)*RecommendedSteps[RealStep];
                 ABegin:=UnderlayInt((FRealEndValue-RealShifting)/step)*step;
                 AEnd:=UpperlayInt((FRealBeginValue-RealShifting)/step)*step;
                 AValue:=Abegin;
                 scale:=(height-BeginShift-EndShift)/(-FRealEndValue+FRealBeginValue);
                 x:=width div 2;
                 while AValue<=AEnd do
                   begin
                     y:=Height-1-round(Height-1-endShift-scale*(AValue-FRealEndValue+RealShifting));
                     DrawMainTickAt(x,y);

                     for i:=1 to FMinorTicks do
                       begin
                         dd:=round(scale*step*i/(FMinorTicks+1));
                         DrawMinorTickAt(x,y+dd);
                       end;
                     s:=format('%5.2f',[AValue/power(10,Realpower)]);
                     if (y<=(height-FBeginShift)) and (y>=FEndShift) then
                     canvas.TextOut(x+dh,y,s);
                     AValue:=AValue+step;
                   end;
               end;
             DrawMainInfoString;
            end;
        end;
        //DrawMainTickAt
      end;
  end;//case FScaleStyle
  canvas.brush.Style:=bsSolid;
end;
procedure IAChartBaseAxisClass.RotateFont(AFont: TFont; ADeg: Integer);
var
  MyLFont : LogFONT;
begin
  GETOBJECT(AFont.Handle,sizeof(MyLFont),@MyLFont);
  MyLfont.lfEscapement:=ADeg*10;
  MyLFont.lfOrientation:=ADeg*10;
  AFont.Handle:=CREATEFONTINDIRECT(MyLFont);//As i think it will automatically delete previouse object...
  //as windows delete object...
end;
procedure IAChartBaseAxisClass.Recalibrate;
var
  trypower,shiftpower : integer;
  i,N,Nb : integer;
begin
  case FScaleStyle of
    iacbasLinear :
      begin
        if FBeginValue<>FEndValue then
          begin
            trypower:=GetRealPower(FEndValue-FbeginValue);
            RealStep:=low(RecommendedSteps);
            Nb:=trunc(abs(FEndValue-FbeginValue)/(power(10,trypower)*RecommendedSteps[low(RecommendedSteps)])+1);
            for i:=low(RecommendedSteps)+1 to high(RecommendedSteps) do
              begin
                N:=trunc(abs(FEndValue-FbeginValue)/(power(10,trypower)*RecommendedSteps[i])+1);
                if abs(N-FWantedTicks)<abs(Nb-FWantedTicks) then
                  begin
                    Nb:=N;
                    RealStep:=i;
                  end;
              end;
            //we defined the real step...
           RealPower:=trypower;
           //and real power..
           //how we define the shifting
           shiftpower:=trypower+2;
           if FbeginValue<FEndValue then
             begin
               if (GetRealPower(FbeginValue)>=shiftpower) and (FbeginValue<>0) then
                 begin
                   RealShifting:=UnderlayInt(FbeginValue/power(10,shiftpower))*power(10,shiftpower);
                 end else RealShifting:=0;
             end else
             begin
               if (GetRealPower(FEndValue)>=shiftpower) and (FEndValue<>0) then
                 begin
                   RealShifting:=UnderlayInt(FEndValue/power(10,shiftpower))*power(10,shiftpower);
                 end else RealShifting:=0;
             end;
            FRealBeginValue :=FBeginValue;
            FRealEndValue:=FEndvalue; 
          end else
          begin
            if FBeginValue<>0 then
              begin
                trypower:=GetRealPower(FbeginValue);
                RealStep:=low(RecommendedSteps);
                Nb:=trunc(abs(FbeginValue)/(power(10,trypower)*RecommendedSteps[low(RecommendedSteps)])+1);
                for i:=low(RecommendedSteps)+1 to high(RecommendedSteps) do
                  begin
                    N:=trunc(abs(FbeginValue)/(power(10,trypower)*RecommendedSteps[i])+1);
                    if abs(N-FWantedTicks)<abs(Nb-FWantedTicks) then
                      begin
                        Nb:=N;
                        RealStep:=i;
                      end;
                  end;
                //we defined the real step...
               RealPower:=trypower;
               //and real power..
               //how we define the shifting
               RealShifting:=0;
               FRealBeginValue :=FBeginValue-FbeginValue/2;
               FRealEndValue:=FBeginValue+FbeginValue/2;
              end else
              begin
                //as above for values=1.0
                trypower:=GetRealPower(1);
                RealStep:=low(RecommendedSteps);
                Nb:=trunc(abs(1)/(power(10,trypower)*RecommendedSteps[low(RecommendedSteps)])+1);
                for i:=low(RecommendedSteps)+1 to high(RecommendedSteps) do
                  begin
                    N:=trunc(abs(1)/(power(10,trypower)*RecommendedSteps[i])+1);
                    if abs(N-FWantedTicks)<abs(Nb-FWantedTicks) then
                      begin
                        Nb:=N;
                        RealStep:=i;
                      end;
                  end;
                //we defined the real step...
               RealPower:=trypower;
               //and real power..
               //how we define the shifting
               RealShifting:=0;
               FRealBeginValue :=-0.5;
               FRealEndValue:=0.5;
              end;
          end;
//        RealPower:=trunc
      end;
  end;
  if assigned(parent) and not (csCreating  in Parent.ControlState)  and IsCreated and parent.HandleAllocated
      then
        begin
          postmessage(parent.handle,CM_AXISRECALIBRATED,cardinal(self),0);
        end;
  //CM_AXISRECALIBRATED
end;

procedure IAChartBaseAxisClass.SetAxisType(Value: IAChartBaseAxisType);
begin
  if Value<>FAxisType then
    begin
      FAxisType := Value;
      case value of
        iacbatHorizontal :
          begin
            SetBounds(FCenterPointLeft,FCenterPointTop-Width div 2,Height,Width);
          end;
        iacbatVertical :
          begin
            SetBounds(FCenterPointLeft-Height div 2,FCenterPointTop-Width,Height,Width);
          end;
      end;
      if assigned(parent) and not (csCreating  in Parent.ControlState)  and IsCreated and
          parent.HandleAllocated
      then
        begin
          postmessage(parent.handle,CM_ORIENTATIONCHANGED,cardinal(self),0);
          postmessage(parent.handle,CM_AXISRECALIBRATED,cardinal(self),0);
        end;
      invalidate;
    end;
end;

procedure IAChartBaseAxisClass.SetBeginShift(Value: Integer);
begin
  if (Value<>FBeginShift) and (Value>=0) and CorrectlySizes(Value,FEndShift) then
    begin
      FBeginShift := Value;
      if assigned(parent) and not (csCreating  in Parent.ControlState) and IsCreated and
          parent.HandleAllocated
          then postmessage(parent.handle,CM_AXISSIZED,cardinal(self),0);
      invalidate;
    end;
end;

procedure IAChartBaseAxisClass.SetBeginValue(Value: Double);
begin
  if FBeginValue<>Value then
    begin
      FBeginValue := Value;
      recalibrate;
      invalidate;
    end;
end;

procedure IAChartBaseAxisClass.SetCenterPointLeft(Value: integer);
begin
  if (Value>=0) and (Value<>FCenterPointLeft) then
    begin
      FCenterPointLeft := Value;
      case FAxisType of
        iacbatHorizontal :
          begin
            SetBounds(FCenterPointLeft,FCenterPointTop-height div 2,Width,Height);
          end;
        iacbatVertical :
          begin
            SetBounds(FCenterPointLeft-Width div 2,FCenterPointTop-Height,Width,Height);
          end;
      end;
    end;
end;

procedure IAChartBaseAxisClass.SetCenterPointTop(Value: Integer);
begin
  if (Value>=0) and (Value<>FCenterPointTop) then
    begin
      FCenterPointTop := Value;
      case FAxisType of
        iacbatHorizontal :
          begin
            SetBounds(FCenterPointLeft,FCenterPointTop-height div 2,Width,Height);
          end;
        iacbatVertical :
          begin
            SetBounds(FCenterPointLeft-Width div 2,FCenterPointTop-Height,Width,Height);
          end;
      end;
    end;
end;

procedure IAChartBaseAxisClass.SetColor(Value: TColor);
begin
  if Value<>(inherited color) then
    begin
      inherited Color:=Value;
      invalidate;
    end;
end;

procedure IAChartBaseAxisClass.SetCurrImageList(Value: TImageList);
begin
  FCurrImageList := Value;
  invalidate;
end;

procedure IAChartBaseAxisClass.SetEndShift(Value: Integer);
begin
  if (Value<>FEndShift) and (Value>=0) and CorrectlySizes(FBeginShift,Value) then
    begin
      FEndShift := Value;
      if assigned(parent) and not (csCreating  in Parent.ControlState) and IsCreated and
          parent.HandleAllocated
          then postmessage(parent.handle,CM_AXISSIZED,cardinal(self),0);
      invalidate;
    end;
end;

procedure IAChartBaseAxisClass.SetEndValue(Value: Double);
begin
  if FEndValue<>Value then
    begin
      FEndValue := Value;
      recalibrate;
      invalidate;
    end;
end;

procedure IAChartBaseAxisClass.SetFont(Value: TFont);
begin
  if assigned(value) then
    begin
      inherited Font.Assign(value);
      IfFontChanged(self);
    end;
end;

procedure IAChartBaseAxisClass.SetHeight(Value: integer);
begin
  if (inherited Height)<>Value then
    begin
      case FAxisType of
        iacbatHorizontal :
          begin
            SetBounds(FCenterPointLeft,FCenterPointTop-Value div 2,Width,Value);
          end;
        iacbatVertical :
          begin
            SetBounds(FCenterPointLeft-Width div 2,FCenterPointTop-Value,Width,Value);
          end;
      end;
    end;
end;

procedure IAChartBaseAxisClass.SetLeft(Value: integer);
begin
  if (inherited Left)<>Value then
    begin
      inherited Left:=Value;
      case FAxisType of
        iacbatHorizontal :
          begin
            FCenterPointLeft:=Value;
          end;
        iacbatVertical :
          begin
            FCenterPointLeft:=Value+Width div 2;
          end;
      end;
    end;
end;

procedure IAChartBaseAxisClass.SetMinorTicks(Value: byte);
begin
  if FMinorTicks<>Value then
    begin
      FMinorTicks := Value;
      invalidate;
    end;
end;

procedure IAChartBaseAxisClass.SetScaleStyle(Value: IAChartBaseAxisStyle);
begin
  if FScaleStyle<>Value then
    begin
      FScaleStyle := Value;
      Recalibrate;
    end;
end;

procedure IAChartBaseAxisClass.SetTop(Value: integer);
begin
  if (inherited Top)<>Value then
    begin
      inherited top:=Value;
      case FAxisType of
        iacbatHorizontal :
          begin
            FCenterPointTop:=Top+height div 2;
          end;
        iacbatVertical :
          begin
            FCenterPointTop:=Top+height;
          end;
      end;
    end;
end;

procedure IAChartBaseAxisClass.SetWantedTicks(Value: byte);
begin
  if Value>=4 then
    begin
      FWantedTicks := Value;
      recalibrate;
      invalidate;
    end;
end;

procedure IAChartBaseAxisClass.SetWidth(Value: integer);
begin
  if (inherited Width)<>Value then
    begin
      case FAxisType of
        iacbatHorizontal :
          begin
            SetBounds(FCenterPointLeft,FCenterPointTop-height div 2,Value,Height);
          end;
        iacbatVertical :
          begin
            SetBounds(FCenterPointLeft-Value div 2,FCenterPointTop-Height,Value,Height);
          end;
      end;
    end;
end;

function IAChartBaseAxisClass.UnderlayInt(AValue: Double): integer;
var SValue : Extended;
begin
  zeromemory(@SValue,10);
  SValue:=AVAlue;
  if SValue<0 then
    begin
      result:=Trunc(SValue)-1;
    end else
    begin
      result:=Trunc(SValue);
    end;
  //if frac(SValue)=0.0 then result:=result-1;
end;

function IAChartBaseAxisClass.UpperlayInt(Avalue: Double): integer;
var SValue : Extended;
begin
  zeromemory(@SValue,10);
  SValue:=AVAlue;
  if SValue<0 then
    begin
      result:=Trunc(SValue);
    end else
    begin
      result:=Trunc(SValue)+1;
    end;
  //if frac(SValue)=0.0 then result:=result+1;
end;

procedure IAChartBaseAxisClass.WMLButtonDblClk(var Message: TWMLButtonDblClk);
begin
  inherited;
  if FAutoEditable then
    begin
      IAChartBaseAisEditWindow:=TIAChartBaseAisEditWindow.Create(self);
      IAChartBaseAisEditWindow.FCurrControl:=self;
      IAChartBaseAisEditWindow.ShowModal;
      case IAChartBaseAisEditWindow.ModalResult of
        mrNone,mrCancel :
          begin
            IAChartBaseAisEditWindow.release;
          end;
        mrOk   :
          begin
            IAChartBaseAisEditWindow.BackUpdate;
            IAChartBaseAisEditWindow.release;
          end;
        else
          begin
            IAChartBaseAisEditWindow.release;
          end;
      end;
    end;
  if assigned(parent) and parent.HandleAllocated
  then postmessage(parent.handle,CM_AXISDBLCLICKED,cardinal(self),0);

end;

procedure IAChartBaseAxisClass.WMLButtonDown(var Message: TWMLButtonDown);
begin
  inherited;
  if assigned(parent) and parent.HandleAllocated
  then postmessage(parent.handle,CM_AXISCLICKED,cardinal(self),0);
end;

procedure IAChartBaseAxisClass.WMLButtonUp(var Message: TWMLButtonUp);
begin
  inherited;
end;

procedure IAChartBaseAxisClass.WMMouseMove(var Message: TWMMouseMove);
begin
  inherited;
end;



procedure IAChartBaseAxisClass.SetBounds(ALeft, ATop, AWidth,  AHeight: Integer);
begin
  if ((Inherited Width)<>AWidth) or ((inherited Height) <>AHeight) then
    begin
      if ((Inherited Top)<>Atop) or ((inherited Left) <>ALeft) then
        begin
          inherited SetBounds(ALeft, ATop, AWidth, AHeight);
          if assigned(parent) and not (csCreating  in Parent.ControlState) and IsCreated and
          parent.HandleAllocated
          then postmessage(parent.handle,CM_AXISSIZED,cardinal(self),0);
          if assigned(parent) and not (csCreating  in Parent.ControlState)  and IsCreated and
          parent.HandleAllocated
          then postmessage(parent.handle,CM_AXISMOVED,cardinal(self),0);
        end else
        begin
          inherited SetBounds(ALeft, ATop, AWidth, AHeight);
          if assigned(parent) and not (csCreating  in Parent.ControlState) and IsCreated and
          parent.HandleAllocated
          then postmessage(parent.handle,CM_AXISSIZED,cardinal(self),0);
        end;
    end else
    begin
      if ((Inherited Top)<>Atop) or ((inherited Left) <>ALeft) then
        begin
          inherited SetBounds(ALeft, ATop, AWidth, AHeight);
          if assigned(parent) and not (csCreating  in Parent.ControlState)  and IsCreated and
          parent.HandleAllocated
          then postmessage(parent.handle,CM_AXISMOVED,cardinal(self),0);
        end else
        begin
          inherited SetBounds(ALeft, ATop, AWidth, AHeight);
        end;
    end;
end;

function IAChartBaseAxisClass.GetVisible: Boolean;
begin
  result:=inherited Visible;
end;

procedure IAChartBaseAxisClass.SetVisible(Value: Boolean);
begin
  if (inherited Visible)<>Value then
    begin
      inherited Visible:=Value;
      if assigned(parent) and not (csCreating  in Parent.ControlState)  and IsCreated and parent.HandleAllocated
      then
        begin
          postmessage(parent.handle,CM_AXISRECALIBRATED,cardinal(self),0);
        end;
      invalidate;
    end;
end;

procedure IAChartBaseAxisClass.SetAutoEditable(Value: Boolean);
begin
  FAutoEditable := Value;
end;

{ IAChartBaseAxisClassData }

procedure IAChartBaseAxisClassData.Assign(Source: TPersistent);
begin
  if Source is IAChartBaseAxisClassData then
    begin
      AxisType:=(Source as IAChartBaseAxisClassData).AxisType;
      Width:=(Source as IAChartBaseAxisClassData).Width;
      Height:=(Source as IAChartBaseAxisClassData).Height;
      Top:=(Source as IAChartBaseAxisClassData).Top;
      Left:=(Source as IAChartBaseAxisClassData).Left;
      CenterPointLeft:=(Source as IAChartBaseAxisClassData).CenterPointLeft;
      BeginShift:=(Source as IAChartBaseAxisClassData).BeginShift;
      CenterPointTop:=(Source as IAChartBaseAxisClassData).CenterPointTop;
      EndShift:=(Source as IAChartBaseAxisClassData).EndShift;


      ScaleStyle:=(Source as IAChartBaseAxisClassData).ScaleStyle;
      MinorTicks:=(Source as IAChartBaseAxisClassData).MinorTicks;
      WantedTicks:=(Source as IAChartBaseAxisClassData).WantedTicks;
      EndValue:=(Source as IAChartBaseAxisClassData).EndValue;
      BeginValue:=(Source as IAChartBaseAxisClassData).BeginValue;
      CurrImageList:=(Source as IAChartBaseAxisClassData).CurrImageList;
      Color:=(Source as IAChartBaseAxisClassData).Color;
      Visible:=(Source as IAChartBaseAxisClassData).Visible;

      Font:=(Source as IAChartBaseAxisClassData).Font;

      AutoEditable:=(Source as IAChartBaseAxisClassData).AutoEditable;
    end else
    begin
      inherited Assign(Source);
    end;
end;

constructor IAChartBaseAxisClassData.Create(AOwner: TComponent);
begin
  inherited Create;
  FAxisField := IAChartBaseAxisClass.Create(AOwner as TWinControl);
  FAxisField.Parent:=AOwner as TWinControl;
end;

destructor IAChartBaseAxisClassData.Destroy;
begin
  FAxisField.free;
  inherited Destroy;
end;

function IAChartBaseAxisClassData.GetAutoEditable: Boolean;
begin
  result:=FAxisField.AutoEditable;
end;

function IAChartBaseAxisClassData.GetAxisType: IAChartBaseAxisType;
begin
  result:=FAxisField.AxisType;
end;

function IAChartBaseAxisClassData.GetBeginShift: Integer;
begin
  result:=FAxisField.BeginShift;
end;

function IAChartBaseAxisClassData.GetBeginValue: Double;
begin
  result:=FAxisField.BeginValue;
end;

function IAChartBaseAxisClassData.GetCenterPointLeft: integer;
begin
  result:=FAxisField.CenterPointLeft;
end;

function IAChartBaseAxisClassData.GetCenterPointTop: Integer;
begin
  result:=FAxisField.CenterPointTop;
end;

function IAChartBaseAxisClassData.GetColor: TColor;
begin
  result:=FAxisField.Color;
end;

function IAChartBaseAxisClassData.GetCurrImageList: TImageList;
begin
  result:=FAxisField.CurrImageList;
end;

function IAChartBaseAxisClassData.GetCurrStep: Double;
begin
  case ScaleStyle of
    iacbasLinear :
      begin
        result:=RecommendedSteps[FAxisField.RealStep]*power(10,FAxisField.RealPower);
      end;
    else
      begin
        result:=0;
      end;
  end;
end;

function IAChartBaseAxisClassData.GetEndShift: Integer;
begin
  result:=FAxisField.EndShift;
end;

function IAChartBaseAxisClassData.GetEndValue: Double;
begin
  result:=FAxisField.EndValue;
end;

function IAChartBaseAxisClassData.GetFont: TFont;
begin
  result:=FAxisField.font;
end;

function IAChartBaseAxisClassData.GetHeight: integer;
begin
  result:=FAxisField.Height;
end;

function IAChartBaseAxisClassData.GetLeft: integer;
begin
  result:=FAxisField.left;
end;

function IAChartBaseAxisClassData.GetMinorTicks: byte;
begin
  result:=FAxisField.MinorTicks;
end;

function IAChartBaseAxisClassData.GetRealBeginValue: Double;
begin
  result:=FAxisField.FRealBeginValue;
end;

function IAChartBaseAxisClassData.GetRealEndvalue: Double;
begin
  result:=FAxisField.FRealEndValue;
end;

function IAChartBaseAxisClassData.GetScaleStyle: IAChartBaseAxisStyle;
begin
  result:=FAxisField.ScaleStyle;
end;

function IAChartBaseAxisClassData.GetTop: integer;
begin
  result:=FAxisField.Top;
end;


function IAChartBaseAxisClassData.GetVisible: Boolean;
begin
  result:=FAxisField.Visible;
end;

function IAChartBaseAxisClassData.GetWantedTicks: byte;
begin
  result:=FAxisField.WantedTicks;
end;

function IAChartBaseAxisClassData.GetWidth: integer;
begin
  result:=FAxisField.Width;
end;

function IAChartBaseAxisClassData.IsYours(
  AObject: IAChartBaseAxisClass): boolean;
begin
    if AObject=FAxisField then result:=true else result:=false;
end;

procedure IAChartBaseAxisClassData.SetAutoEditable(Value: Boolean);
begin
  FAxisField.AutoEditable:=Value;
end;

procedure IAChartBaseAxisClassData.SetAxisType(Value: IAChartBaseAxisType);
begin
  FAxisField.AxisType:=Value;
end;

procedure IAChartBaseAxisClassData.SetBeginShift(Value: Integer);
begin
  FAxisField.BeginShift:=Value;
end;

procedure IAChartBaseAxisClassData.SetBeginValue(Value: Double);
begin
  FAxisField.BeginValue:=Value;
end;



procedure IAChartBaseAxisClassData.SetCenterPointLeft(Value: integer);
begin
  FAxisField.CenterPointLeft:=Value;
end;

procedure IAChartBaseAxisClassData.SetCenterPointTop(Value: Integer);
begin
  FAxisField.CenterPointTop:=Value;
end;

procedure IAChartBaseAxisClassData.SetColor(Value: TColor);
begin
  FAxisField.Color:=Value;
end;

procedure IAChartBaseAxisClassData.SetCurrImageList(Value: TImageList);
begin
  FAxisField.CurrImageList:=Value;
end;

procedure IAChartBaseAxisClassData.SetEndShift(Value: Integer);
begin
  FAxisField.EndShift:=Value;
end;

procedure IAChartBaseAxisClassData.SetEndValue(Value: Double);
begin
  FAxisField.EndValue:=Value;
end;

procedure IAChartBaseAxisClassData.SetFont(Value: TFont);
begin
  FAxisField.Font:=Value;
end;

procedure IAChartBaseAxisClassData.SetHeight(Value: integer);
begin
  FAxisField.Height:=Value;
end;

procedure IAChartBaseAxisClassData.SetLeft(Value: integer);
begin
  FAxisField.Left:=Value;
end;

procedure IAChartBaseAxisClassData.SetLength(AValue: integer);
begin
  case FAxisField.FAxisType of
    iacbatHorizontal :
      begin
        FAxisField.width:=AValue;
      end;
    iacbatVertical   :
      begin
        FAxisField.height:=AValue;
      end;
  end;
end;

procedure IAChartBaseAxisClassData.SetMinorTicks(Value: byte);
begin
  FAxisField.MinorTicks:=Value;
end;

procedure IAChartBaseAxisClassData.SetScaleStyle(Value: IAChartBaseAxisStyle);
begin
  FAxisField.ScaleStyle:=Value;
end;

procedure IAChartBaseAxisClassData.SetTop(Value: integer);
begin
  FAxisField.Top:=Value;
end;


procedure IAChartBaseAxisClassData.SetVisible(Value: Boolean);
begin
  FAxisField.Visible:=Value;
end;

procedure IAChartBaseAxisClassData.SetWantedTicks(Value: byte);
begin
  FAxisField.WantedTicks:=Value;
end;

procedure IAChartBaseAxisClassData.SetWidth(Value: integer);
begin
  FAxisField.Width:=Value;
end;

end.
