unit IAChartBaseMovingUnit;

interface

uses Windows, Messages,Controls,SysUtils, Classes,Graphics;
const
  CM_USERSELECTED = CM_BASE+200;
  CM_USERMURESIZED =CM_BASE+201;
  CM_USERDBLCLICK = CM_BASE+202;

type
  IAChartFixType = (iacftNone,iacftMain,iacftLeftTop,iacftLeft,iacftLeftBottom,
                                        iacftRightTop,iacftRight,iacftRightBottom,
                                        iacftTop,iacftBottom);
  IAChartBaseMovingClass = class(TGraphicControl)
  private
    IsMouseDown : Boolean;
    IsMoved : Boolean;
    IsChartFixType :IAChartFixType;
    LastX,LastY : Integer;
    FTransparence: Boolean;
    FSizeAble: Boolean;
    FMoveAble: Boolean;
    FCanMultySelect: Boolean;
    FSelectable: Boolean;
    procedure WMLButtonDown(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMLButtonDblClk(var Message: TWMLButtonDblClk); message WM_LBUTTONDBLCLK;
    procedure WMMouseMove(var Message: TWMMouseMove); message WM_MOUSEMOVE;
    procedure WMLButtonUp(var Message: TWMLButtonUp); message WM_LBUTTONUP;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMUSERSELECTED(var Message: TMessage); message CM_USERSELECTED;

    procedure setProperlyCursor;
    procedure SetTransparence(Value: Boolean);
    procedure SetMoveAble(Value: Boolean);
    procedure SetSizeAble(Value: Boolean);
    procedure SetCanMultySelect(Value: Boolean);
    procedure SetSelectable(Value: Boolean);
  protected
    IsSelected : Boolean;
    procedure Paint; override;
  public
    
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure SetSelection(AValue : Boolean);
    procedure Assign(Source: TPersistent); override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    property Canvas;
    property Transparence : Boolean read FTransparence write SetTransparence;
    property SizeAble : Boolean read FSizeAble write SetSizeAble;
    property MoveAble : Boolean read FMoveAble write SetMoveAble;
    property CanMultySelect : Boolean read FCanMultySelect write SetCanMultySelect;
    property Selectable : Boolean read FSelectable write SetSelectable;
  end;

implementation

{ IAChartBaseMovingClass }

procedure IAChartBaseMovingClass.Assign(Source: TPersistent);
begin
  if Source is IAChartBaseMovingClass then
    begin
      SizeAble:=(Source as IAChartBaseMovingClass).SizeAble;
      MoveAble:=(Source as IAChartBaseMovingClass).MoveAble;
      Transparence:=(Source as IAChartBaseMovingClass).Transparence;
      Selectable:=(Source as IAChartBaseMovingClass).Selectable;
      (self as TGraphicControl).Assign((Source as TGraphicControl));
    end else
    begin
      inherited Assign(Source);
    end;
end;


procedure IAChartBaseMovingClass.CMMouseEnter(var Message: TMessage);
begin
  inherited;
end;

procedure IAChartBaseMovingClass.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  
end;

procedure IAChartBaseMovingClass.CMUSERSELECTED(var Message: TMessage);
var i : integer;
begin
  if message.WParam<>0 then
    begin
      if (parent<>nil)and assigned(parent) then
        begin
          for i:=0 to parent.controlcount-1 do
            begin
              if (parent.Controls[i] is IAChartBaseMovingClass) and
                 ((parent.Controls[i] as IAChartBaseMovingClass)<>self) and
                 (parent.Controls[i] as IAChartBaseMovingClass).isselected and
                 (not(parent.Controls[i] as IAChartBaseMovingClass).CanMultySelect)
                 then
                 begin
                   (parent.Controls[i] as IAChartBaseMovingClass).SetSelection(false);
                 end;
            end;
        end;
    end;
end;

constructor IAChartBaseMovingClass.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  IsChartFixType:=iacftNone;
  width:=20;
  height:=20;
  LastX:=0;
  LastY:=0;
  IsSelected:=false;
  FSelectable:=True;
  FMoveAble:=TRUE;
  FSizeAble:=TRUE;
  self.ControlStyle:=self.ControlStyle+[csOpaque];
  FTransparence:=false;
  FCanMultySelect:=false;
end;

destructor IAChartBaseMovingClass.Destroy;
begin
  inherited Destroy;
end;

procedure IAChartBaseMovingClass.Paint;
var i : integer;
begin
  if not Visible then exit;
  if IsSelected then
    begin
      canvas.pen.Color:=clBlack;
      canvas.pen.Mode:=pmNot;
      Canvas.Pen.Width:=5;
      canvas.moveto(0+2,0+2);
      canvas.lineto(width-1-2,0+2);
      canvas.lineto(width-1-2,height-1-2);
      canvas.lineto(0+2,height-1-2);
      canvas.lineto(0+2,0+2);

      canvas.pen.Mode:=pmCopy;
      Canvas.Pen.Width:=1;
      canvas.brush.color:=clBlack;
      canvas.brush.Style:=bsBDiagonal;
      for i:=0 to 4 do
        begin
          canvas.FrameRect(rect(i,i,width-1-i,height-1-i));
        end;
      canvas.brush.Style:=bsSolid;  
    end;
end;

procedure IAChartBaseMovingClass.SetBounds(ALeft, ATop, AWidth,
  AHeight: Integer);
begin
    if ((Inherited Width)<>AWidth) or ((inherited Height) <>AHeight) then
    begin
      inherited SetBounds(ALeft, ATop, AWidth, AHeight);
      if assigned(parent) and not (csCreating  in Parent.ControlState) and  parent.HandleAllocated
      then
        begin
          if isselected then  postmessage(parent.handle,CM_USERMURESIZED,cardinal(self),0) else
                              postmessage(parent.handle,CM_USERMURESIZED,cardinal(self),1);
        end;
    end else
    begin
      inherited SetBounds(ALeft, ATop, AWidth, AHeight);
    end;
end;

procedure IAChartBaseMovingClass.SetCanMultySelect(Value: Boolean);
begin
  if FCanMultySelect<>Value then
    begin
      FCanMultySelect := Value;
      SetSelection(false);
    end;
end;

procedure IAChartBaseMovingClass.SetMoveAble(Value: Boolean);
begin
  if FMoveAble <> Value then  FMoveAble := Value;
end;

procedure IAChartBaseMovingClass.setProperlyCursor;
begin
if Isselected then
  begin
  case IsChartFixType of
    iacftMain :
      begin
        if FMoveAble then self.Cursor:=crDrag else self.Cursor:=crDefault;
      end;
    iacftLeftTop :
      begin
        if FSizeAble then self.Cursor:=crSizeNWSE else self.Cursor:=crDefault;
      end;
    iacftLeft :
      begin
        if FSizeAble then self.Cursor:=crSizeWE else self.Cursor:=crDefault;
      end;
    iacftLeftBottom :
      begin
        if FSizeAble then self.Cursor:=crSizeNESW else self.Cursor:=crDefault;
      end;
    iacftRightTop :
      begin
        if FSizeAble then self.Cursor:=crSizeNESW else self.Cursor:=crDefault;
      end;
    iacftRight :
      begin
        if FSizeAble then self.Cursor:=crSizeWE else self.Cursor:=crDefault;
      end;
    iacftRightBottom :
      begin
        if FSizeAble then self.Cursor:=crSizeNWSE else self.Cursor:=crDefault;
      end;
    iacftTop :
      begin
        if FSizeAble then self.Cursor:=crSizeNS else self.Cursor:=crDefault;
      end;
    iacftBottom :
      begin
        if FSizeAble then self.Cursor:=crSizeNS else self.Cursor:=crDefault;
      end;
  end;
  end else
  begin
    self.Cursor:=crDefault;
  end;
end;

procedure IAChartBaseMovingClass.SetSelectable(Value: Boolean);
begin
  if FSelectable<>Value then
    begin
      FSelectable := Value;
      if not FSelectable then IsSelected:=false;
      invalidate;
    end;
end;

procedure IAChartBaseMovingClass.SetSelection(AValue: Boolean);
begin
  if (IsSelected<>AValue) and FSelectable then
    begin
      IsSelected:=AValue;
      if IsSelected and not FCanMultySelect then
        begin
          perform(CM_USERSELECTED,1,0);
        end;
      invalidate;
    end;
end;

procedure IAChartBaseMovingClass.SetSizeAble(Value: Boolean);
begin
 if  FSizeAble <> Value then FSizeAble := Value;
end;

procedure IAChartBaseMovingClass.SetTransparence(Value: Boolean);
begin
  if FTransparence <> Value then
    begin
      FTransparence := Value;
      if Value then self.ControlStyle:=self.ControlStyle-[csOpaque]
               else self.ControlStyle:=self.ControlStyle+[csOpaque];
      invalidate;
    end;
end;

procedure IAChartBaseMovingClass.WMLButtonDblClk(var Message: TWMLButtonDblClk);
begin
  inherited;
  ISMouseDown:=false;
  postmessage(parent.handle,CM_USERDBLCLICK,cardinal(self),0);
  invalidate;
end;

procedure IAChartBaseMovingClass.WMLButtonDown(var Message: TWMLButtonDown);
var
  Ax,Ay : integer;
begin
  inherited;
  AX:=Mouse.CursorPos.x-ClientOrigin.x;
  AY:=Mouse.CursorPos.y-ClientOrigin.y;
  //iacftNone,iacftMain,iacftLeftTop,iacftLeft,iacftLeftBottom,
  //                                      iacftRightTop,iacftRight,iacftRightBottom,
  //                                      iacftTop,iacftBottom
  if ptinrect(self.ClientRect,point(Message.XPos,Message.YPos)) then
    begin
      if AX<5 then
        begin
          if AY<5 then
            begin
              IsChartFixType:=iacftLeftTop;
            end else
            begin
              if AY>(height-1-5) then
                begin
                  IsChartFixType:=iacftLeftBottom;
                end else
                begin
                  IsChartFixType:=iacftLeft;
                end;
            end;
        end else
        begin
          if AX>(width-1-5) then
            begin
              if AY<5 then
                begin
                  IsChartFixType:=iacftRightTop;
                end else
                begin
                  if AY>(height-1-5) then
                    begin
                      IsChartFixType:=iacftRightBottom;
                    end else
                    begin
                      IsChartFixType:=iacftRight;
                    end;
                end;
            end else
            begin
              if AY<5 then
                begin
                  IsChartFixType:=iacftTop;
                end else
                begin
                  if AY>(height-1-5) then
                    begin
                      IsChartFixType:=iacftBottom;
                    end else
                    begin
                      IsChartFixType:=iacftMain;
                    end;
                end;
            end;
        end;
    end else
    begin
      IsChartFixType:=iacftNone;
    end;
  setProperlyCursor;

  ISMouseDown:=True;
  IsMoved:=false;
  LastX:=Mouse.CursorPos.x-ClientOrigin.x;
  LastY:=Mouse.CursorPos.y-ClientOrigin.y;
  invalidate;
end;

procedure IAChartBaseMovingClass.WMLButtonUp(var Message: TWMLButtonUp);
begin
  inherited;
  if IsMoved then
    begin
      IsSelected:= TRue;
    end else
    begin
      IsSelected:= not IsSelected;
    end;
  if not FSelectable then Isselected:=false;
  if  Isselected and (  (LastX<>(Mouse.CursorPos.x-ClientOrigin.x)) or (LastY<>(Mouse.CursorPos.y-ClientOrigin.y))) then
    begin
      Isselected:=false;
    end;
  if (IsSelected or (not FSelectable))and not FCanMultySelect then
    begin
      perform(CM_USERSELECTED,1,0);
    end;
  ISMouseDown:=false;
  setProperlyCursor;
  invalidate;
end;

procedure IAChartBaseMovingClass.WMMouseMove(var Message: TWMMouseMove);
var
AX,AY : integer;
begin
  inherited;
    AX:=Mouse.CursorPos.x-ClientOrigin.x;
  AY:=Mouse.CursorPos.y-ClientOrigin.y;
  if not IsMouseDown then
    begin
      if ptinrect(self.ClientRect,point(Message.XPos,Message.YPos)) then
        begin
          if AX<5 then
            begin
              if AY<5 then
                begin
                  IsChartFixType:=iacftLeftTop;
                end else
                begin
                  if AY>(height-1-5) then
                    begin
                      IsChartFixType:=iacftLeftBottom;
                    end else
                    begin
                      IsChartFixType:=iacftLeft;
                    end;
                end;
            end else
            begin
              if AX>(width-1-5) then
                begin
                  if AY<5 then
                    begin
                      IsChartFixType:=iacftRightTop;
                    end else
                    begin
                      if AY>(height-1-5) then
                        begin
                          IsChartFixType:=iacftRightBottom;
                        end else
                        begin
                          IsChartFixType:=iacftRight;
                        end;
                    end;
                end else
                begin
                  if AY<3 then
                    begin
                      IsChartFixType:=iacftTop;
                    end else
                    begin
                      if AY>(height-1-5) then
                        begin
                          IsChartFixType:=iacftBottom;
                        end else
                        begin
                          IsChartFixType:=iacftMain;
                        end;
                    end;
                end;
            end;
        end else
        begin
          IsChartFixType:=iacftNone;
        end;
      setProperlyCursor;
    end;
  if ISMouseDown and IsSelected then
    begin
      case IsChartFixType of
        iacftNone :
          begin

          end;
        else
          begin
            IsMoved:=True;
            case IsChartFixType of
              iacftMain :
                begin
                  if FMoveAble then
                  SetBounds(left+(Mouse.CursorPos.x-ClientOrigin.x-LastX), top +(Mouse.CursorPos.y-ClientOrigin.y-LastY),
                   width,height);
                  LastX:=Mouse.CursorPos.x-ClientOrigin.x;
                  LastY:=Mouse.CursorPos.y-ClientOrigin.y;

                end;
              iacftLeftTop :
                begin
                  if FSizeAble then
                  SetBounds(left+(Mouse.CursorPos.x-ClientOrigin.x-LastX), top +(Mouse.CursorPos.y-ClientOrigin.y-LastY),
                  width-(Mouse.CursorPos.x-ClientOrigin.x-LastX),height-(Mouse.CursorPos.y-ClientOrigin.y-LastY));
                  LastX:=Mouse.CursorPos.x-ClientOrigin.x;
                  LastY:=Mouse.CursorPos.y-ClientOrigin.y;

                end;
              iacftLeft :
                begin
                  if FSizeAble then
                  SetBounds(left+(Mouse.CursorPos.x-ClientOrigin.x-LastX), top,
                   width-(Mouse.CursorPos.x-ClientOrigin.x-LastX),Height);
                  LastX:=Mouse.CursorPos.x-ClientOrigin.x;
                  LastY:=Mouse.CursorPos.y-ClientOrigin.y;

                end;
              iacftLeftBottom :
                begin
                  if FSizeAble then
                  SetBounds(left+(Mouse.CursorPos.x-ClientOrigin.x-LastX), top,
                  width-(Mouse.CursorPos.x-ClientOrigin.x-LastX),height+(Mouse.CursorPos.y-ClientOrigin.y-LastY));
                  LastX:=Mouse.CursorPos.x-ClientOrigin.x;
                  LastY:=Mouse.CursorPos.y-ClientOrigin.y;
                end;
              iacftRightTop :
                begin
                  if FSizeAble then
                  SetBounds(left, top +(Mouse.CursorPos.y-ClientOrigin.y-LastY),
                  width+(Mouse.CursorPos.x-ClientOrigin.x-LastX),height-(Mouse.CursorPos.y-ClientOrigin.y-LastY));
                  LastX:=Mouse.CursorPos.x-ClientOrigin.x;
                  LastY:=Mouse.CursorPos.y-ClientOrigin.y;
                end;
              iacftRight :
                begin
                  if FSizeAble then
                  SetBounds(left, top,
                  width+(Mouse.CursorPos.x-ClientOrigin.x-LastX),height);
                  LastX:=Mouse.CursorPos.x-ClientOrigin.x;
                  LastY:=Mouse.CursorPos.y-ClientOrigin.y;
                end;
              iacftRightBottom :
                begin
                  if FSizeAble then
                  SetBounds(left, top ,
                  width+(Mouse.CursorPos.x-ClientOrigin.x-LastX),height+(Mouse.CursorPos.y-ClientOrigin.y-LastY));
                  LastX:=Mouse.CursorPos.x-ClientOrigin.x;
                  LastY:=Mouse.CursorPos.y-ClientOrigin.y;
                end;
              iacftTop :
                begin
                  if FSizeAble then
                  SetBounds(left, top +(Mouse.CursorPos.y-ClientOrigin.y-LastY),
                   width,height-(Mouse.CursorPos.y-ClientOrigin.y-LastY));
                  LastX:=Mouse.CursorPos.x-ClientOrigin.x;
                  LastY:=Mouse.CursorPos.y-ClientOrigin.y;

                end;
              iacftBottom :
                begin
                  if FSizeAble then
                  SetBounds(left, top,
                  width,height+(Mouse.CursorPos.y-ClientOrigin.y-LastY));
                  LastX:=Mouse.CursorPos.x-ClientOrigin.x;
                  LastY:=Mouse.CursorPos.y-ClientOrigin.y;
                end;
            end;
          end;
      end;

      invalidate;
    end;
end;

end.
