unit IAChartBaseTextUnit;

interface

uses Windows, Messages,Controls,SysUtils, Classes,Graphics,IAChartBaseMovingUnit,IAChartBaseTextEditUnit;

const SysWords: array[0..13] of String = ('<B>','</B>',
                                         '<I>','</I>',
                                         '<U>','</U>',
                                         '<up>','</up>',
                                         '<down>','</down>',
                                         '<size>','</size>',
                                         '<inline>','</inline>');

type
  IAChartBaseTextOrientation = (iacbto_0_Degrees,iacbto_90_Degrees,iacbto_180_Degrees,iacbto_270_Degrees);
  IAChartBaseBorderStyle = (iacbbsNone,iacbbsSimple,iacbbsRising,iacbbsPressed,iacbbsUp,iacbbsDown);
  IAChartBaseTextClass = class;
  IAChartBaseTextClassData = class(TPersistent)
  private
    FTextField : IAChartBaseTextClass;
    function GetFont: TFont;
    function GetStringsArray: TStringList;
    function GetText: TCaption;
    procedure SetFont(Value: TFont);
    procedure SetStringsArray(Value: TStringList);
    procedure SetText(Value: TCaption);
    function GetTransparent: Boolean;
    procedure SetTransparent(Value: Boolean);
    function GetColor: TColor;
    procedure SetColor(Value: TColor);
    function GetAutoSize: Boolean;
    function GetHeight: integer;
    function GetWidth: integer;
    procedure SetAutoSize(Value: Boolean);
    procedure SetHeight(Value: integer);
    procedure SetWidth(Value: integer);
    function GetTextOrientation: IAChartBaseTextOrientation;
    procedure SetTextOrientation(Value: IAChartBaseTextOrientation);
    function GetBorderStyle: IAChartBaseBorderStyle;
    function GetInnerGap: integer;
    function GetOuterGap: integer;
    procedure SetBorderStyle(Value: IAChartBaseBorderStyle);
    procedure SetInnerGap(Value: integer);
    procedure SetOuterGap(Value: integer);
    function GetCurrImageList: TImageList;
    procedure SetCurrImageList(Value: TImageList);
    function GetLeft: Integer;
    function GetTop: integer;
    procedure SetLeft(Value: Integer);
    procedure SetTop(Value: integer);
    function GetVisible: Boolean;
    procedure SetVisible(Value: Boolean);
    function GetAutoEditable: Boolean;
    procedure SetAutoEditable(Value: Boolean);

  protected

  public
    function IsYours(AObject :IAChartBaseTextClass): boolean;
    procedure Assign(Source: TPersistent); override;
    constructor Create(AOwner: TComponent); virtual;
    destructor Destroy; override;

  published

    property Text: TCaption read GetText write SetText;
    property Font: TFont read GetFont write SetFont;
    property Transparent : Boolean read GetTransparent write SetTransparent;
    property Color  : TColor read GetColor write SetColor;
    property StringsArray : TStringList read GetStringsArray write SetStringsArray;
    property AutoSize : Boolean read GetAutoSize write SetAutoSize;
    property Width : integer read GetWidth write SetWidth;
    property Height : integer read GetHeight write SetHeight;
    property Top : integer read GetTop write SetTop;
    property Left : Integer read GetLeft write SetLeft;
    property TextOrientation : IAChartBaseTextOrientation read GetTextOrientation write SetTextOrientation;
    property BorderStyle : IAChartBaseBorderStyle read GetBorderStyle write SetBorderStyle default iacbbsSimple;
    property InnerGap : integer read GetInnerGap write SetInnerGap default 5;
    property OuterGap : integer read GetOuterGap write SetOuterGap default 5;
    property CurrImageList : TImageList read GetCurrImageList write SetCurrImageList;
    property Visible : Boolean read GetVisible write SetVisible;
    property AutoEditable : Boolean read GetAutoEditable write SetAutoEditable;
  end;
  IAChartBaseTextClass = class(IAChartBaseMovingClass)
  private
    FShiftText : TPoint;
    FSizeText : TSize;
    FStringsArray: TStringList;
    FPosArray : array of TSize;
    FAutoSize: Boolean;
    FTextOrientation: IAChartBaseTextOrientation;
    FBorderStyle: IAChartBaseBorderStyle;
    FOuterGap: integer;
    FInnerGap: integer;
    FCurrImageList: TImageList;
    FAutoEditable: Boolean;
    function GetText: TCaption;
    procedure SetText(Value: TCaption);
    procedure SetStringsArray(const Value: TStringList);
    function GetFont: TFont;
    procedure SetFont(Value: TFont);
    procedure DefineSizeShifting;
    function GetBeforeFirstSys(var s: string): string;
    function GetFirstSys(var s: string): integer;
    function GetColor: TColor;
    procedure SetColor(Value: TColor);
    procedure SetAutoSize(Value: Boolean);
    procedure SetTextOrientation(Value: IAChartBaseTextOrientation);
    procedure DrawOnCanvasText;
    procedure RotateFont(AFont : TFont;ADeg : Integer);
    procedure IfFontChanged(Sender: TObject);
    procedure SetBorderStyle(Value: IAChartBaseBorderStyle);
    procedure SetInnerGap(Value: integer);
    procedure SetOuterGap(Value: integer);
    function GetAllGap: integer;
    function GetDColor(Value: Tcolor): Tcolor;
    function GetLColor(Value: Tcolor): Tcolor;
    function GetAntiColor(Value: Tcolor): Tcolor;
    procedure SetCurrImageList(Value: TImageList);
    function GetVisible: boolean;
    procedure SetVisible(Value: boolean);
    procedure SetAutoEditable(Value: Boolean);
    procedure WMLButtonDblClk(var Message: TWMLButtonDblClk); message WM_LBUTTONDBLCLK;
  protected
    procedure Paint; override;
    procedure Loaded;override;
  public
    procedure Assign(Source: TPersistent); override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property CurrImageList : TImageList read FCurrImageList write SetCurrImageList;
  published

    property Text: TCaption read GetText write SetText;
    property Font: TFont read GetFont write SetFont;
    property Color: TColor read GetColor write SetColor;
    property StringsArray : TStringList read FStringsArray write SetStringsArray;
    property AutoSize : Boolean read FAutoSize write SetAutoSize default True;
    property TextOrientation : IAChartBaseTextOrientation read FTextOrientation write SetTextOrientation
                                default iacbto_0_Degrees;
    property BorderStyle : IAChartBaseBorderStyle read FBorderStyle write SetBorderStyle default iacbbsSimple;
    property InnerGap : integer read FInnerGap write SetInnerGap default 5;
    property OuterGap : integer read FOuterGap write SetOuterGap default 5;
    property Visible : boolean read GetVisible write SetVisible;
    property AutoEditable : Boolean read FAutoEditable write SetAutoEditable;
  end;

implementation

{ IAChartBaseTextClass }

procedure IAChartBaseTextClass.Assign(Source: TPersistent);
begin
  if Source is IAChartBaseTextClass then
    begin
      Text:=(Source as IAChartBaseTextClass).Text;
      Font:=(Source as IAChartBaseTextClass).Font;
      Color:=(Source as IAChartBaseTextClass).Color;
      AutoSize:=(Source as IAChartBaseTextClass).AutoSize;
      TextOrientation:=(Source as IAChartBaseTextClass).TextOrientation;
      InnerGap:=(Source as IAChartBaseTextClass).InnerGap;
      OuterGap:=(Source as IAChartBaseTextClass).OuterGap;
      Borderstyle:=(Source as IAChartBaseTextClass).Borderstyle;
      CurrImageList:=(Source as IAChartBaseTextClass).CurrImageList;
      AutoEditable:=(Source as IAChartBaseTextClass).AutoEditable;
      (self as IAChartBaseMovingClass).assign(Source as IAChartBaseMovingClass);
    end else
    begin
      inherited Assign(Source);
    end;
end;


constructor IAChartBaseTextClass.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FStringsArray:=TStringList.Create;
  FStringsArray.OnChange:=IfFontChanged;
  FStringsArray.Clear;
  FAutoSize:=True;
  FTextOrientation := iacbto_0_Degrees;
  self.ParentFont:=false;
  Font.OnChange:=IfFontChanged;
  Font.Name:='Arial';
  FInnerGap:=5;
  FOuterGap:=5;
  FBorderStyle:=iacbbsSimple;
  FCurrImageList:=nil;
  FAutoEditable:=True;
end;

procedure IAChartBaseTextClass.DefineSizeShifting;
var AFont : TFont;
    AMainRight,ARight,ATop,ABottom,AMainTop,AMainBottom,TryTop,tryBottom,TryRight : Integer;
    ACurrSize,AFirstShift : TSize;
    i : integer;
    s,ds,sysS : string;
    sysNumber : integer;
    IsUp,IsDown,LastUp,LastDown : boolean;
    ABitmap : TBitmap;
    SubSSize : TSize;
    NImage : integer;
begin
  setlength(FPosArray,0);
  setlength(FPosArray,FStringsArray.Count);
  AFont:=TFont.Create;
  AFont.Assign(self.Font);
  ABitmap:=TBitmap.Create;
  ABitmap.PixelFormat:=pf24Bit;
  ABitmap.Width:=100;
  ABitmap.Height:=24;

  ACurrSize.cx:=0;ACurrSize.cy:=0;
  AFirstShift.cx:=0;AFirstShift.cy:=0;
  for i:=0 to FStringsArray.Count-1 do//At All strings
    begin
      s:=FStringsArray[i];  isUp:=false;isDown:=false;LastUp:=false;LastDown:=false;
      AMainRight:=0;AMainTop:=0;AMainBottom:=0;//for mainstring
      ARight:=0;ATop:=0;ABottom:=0;//for all string...
      sysNumber:=-1;
      repeat
        ds:=GetBeforeFirstSys(s);
        if ds='' then
          begin
            sysNumber:=GetFirstSys(s);
            case sysNumber of
              0 :
                begin
                  AFont.Style:=AFont.Style+[fsBold];
                end;
              1 :
                begin
                  AFont.Style:=AFont.Style-[fsBold];
                end;  
              2 :
                begin
                  AFont.Style:=AFont.Style+[fsItalic];
                end;
              3 :
                begin
                  AFont.Style:=AFont.Style-[fsItalic];
                end;
              4 :
                begin
                  AFont.Style:=AFont.Style+[fsUnderline];
                end;
              5 :
                begin
                  AFont.Style:=AFont.Style-[fsUnderline];
                end;
              6 :
                begin
                  isUp:=true;LastUp:=true;
                  isdown:=false;
                end;
              7 :
                begin
                  LastDown:=false;isUp:=false;
                end;
              8 :
                begin
                  isDown:=true;LastDown:=True;
                  isUp:=false;
                end;
              9 :
                begin
                  LastUp:=false;isDown:=false;
                end;
              11 : //here we changing the size...
                begin
                  AFont.Size:=strtointdef(sysS,AFont.Size);
                end;
              13 : //here we inserting the bitmap...
                begin
                  NImage:=strtointdef(sysS,0);
                  if (FCurrImageList<>nil) and assigned(FCurrImageList) and(FCurrImageList.Count>NImage) and (NImage>=0) then
                    begin
                      case FTextOrientation of
                        iacbto_0_Degrees,iacbto_180_Degrees :
                          begin
                            SubSSize.cx:=FCurrImageList.Width;
                            SubSSize.cy:=FCurrImageList.Height;
                          end;
                        iacbto_90_Degrees,iacbto_270_Degrees :
                          begin
                            SubSSize.cx:=FCurrImageList.Height;
                            SubSSize.cy:=FCurrImageList.Width;
                          end;
                       end;   
                      //-- as main text
                      ARight:=ARight+subSSize.cx;
                      AMainRight:=ARight;
                      AMainTop:=AMainTop-(SubSSize.cy-(AMainBottom-AmainTop)) div 2;
                      AMainBottom:=AMainTop+SubSSize.cy;
                      if ATop>AMainTop then ATop:=AMainTop;
                      if ABottom<AMainBottom then ABottom:=AMainBottom;
                    end;
                end;
            end; //End of Case sysnumber
          end else
          begin
            case sysNumber of
              10,12 : //<size><inline>
                begin//that means no! action - so waiting for next
                  sysS:=ds;
                end;
              else
                begin
                  if (not isUp) and (not isDown) then
                    begin
                      LastUp:=false;LastDown:=false;
                      AMainRight:=ARight;
                    end;
                  ABitmap.Canvas.Font.Assign(AFont);
                  GetTextExtentPoint32(ABitmap.Canvas.Handle,pChar(ds),length(ds),SubSSize);
                  if isUp then
                    begin
                      if lastDown then
                        begin
                          TryTop:=AMainTop-(SubSSize.cy div 2);
                          TryBottom:=TryTop+SubSSize.cy;
                          TryRight:=AMainRight+subSSize.cx;
                          if ATop>TryTop then ATop:=TryTop;
                          if Abottom<TryBottom then ABottom:=TryBottom;
                          if ARight<TryRight then ARight:=TryRight;
                          LastDown:=false;
                          AMainRight:=ARight;
                        end else
                        begin
                          TryTop:=AMainTop-(SubSSize.cy div 2);
                          TryBottom:=TryTop+SubSSize.cy;
                          ARight:=AMainRight+subSSize.cx;
                          if ATop>TryTop then ATop:=TryTop;
                          if Abottom<TryBottom then ABottom:=TryBottom;
                        end;
                    end else
                    begin
                      if isDown then
                        begin
                          if LastUp then
                            begin
                              TryTop:=AMainBottom-(SubSSize.cy div 2);
                              TryBottom:=TryTop+SubSSize.cy;
                              TryRight:=AMainRight+subSSize.cx;
                              if ATop>TryTop then ATop:=TryTop;
                              if Abottom<TryBottom then ABottom:=TryBottom;
                              if ARight<TryRight then ARight:=TryRight;
                              LastUp:=false;
                              AMainRight:=ARight;
                            end else
                            begin
                              TryTop:=AMainBottom-(SubSSize.cy div 2);
                              TryBottom:=TryTop+SubSSize.cy;
                              ARight:=AMainRight+subSSize.cx;
                              if ATop>TryTop then ATop:=TryTop;
                              if Abottom<TryBottom then ABottom:=TryBottom;
                            end;
                        end else
                        begin
                          ARight:=ARight+subSSize.cx;
                          AMainRight:=ARight;
                          AMainTop:=AMainTop-(SubSSize.cy-(AMainBottom-AmainTop)) div 2;
                          AMainBottom:=AMainTop+SubSSize.cy;
                          if ATop>AMainTop then ATop:=AMainTop;
                          if ABottom<AMainBottom then ABottom:=AMainBottom;
                        end;
                    end;
                end;
            end; //End of Case sysnumber
          end;
      until (ds='') and (sysNumber=-1);
      if i=0 then
        begin
          FPosArray[i].cx:=0;
          FPosArray[i].cy:=-ATop;
        end else
        begin
          FPosArray[i].cx:=0;
          FPosArray[i].cy:=-ATop+ACurrSize.cy;
        end;
      if i=0 then
        begin
          AFirstShift.cy:=-ATop;
          AFirstShift.cy:=0;
          if AFirstShift.cy<0 then AFirstShift.cy:=0;
        end;
      if ACurrSize.cx<ARight then ACurrSize.cx:=ARight;
      ACurrSize.cy:=ACurrSize.cy+(ABottom-ATop);

    end;
  FShiftText.x:=AFirstShift.cx;FShiftText.y:=AFirstShift.cy;
  FSizeText.cx:=ACurrSize.cx;FSizeText.cy:=ACurrSize.cy;
  ABitmap.free;
  AFont.Free;
  if FAUtoSize then
    begin
      case FTextOrientation of
        iacbto_0_Degrees,iacbto_180_Degrees:
          begin
            width:=FSizeText.cx+2*GetAllGap;
            height:=FSizeText.cy+2*GetAllGap;
          end;
        iacbto_90_Degrees,iacbto_270_Degrees :
          begin
            width:=FSizeText.cy+2*GetAllGap;
            height:=FSizeText.cx+2*GetAllGap;
          end;
      end;
    end;
end;

destructor IAChartBaseTextClass.Destroy;
begin

  setlength(FPosArray,0);
  FStringsArray.Free;
  inherited Destroy;
end;

procedure IAChartBaseTextClass.DrawOnCanvasText;
var AFont,LastFont : TFont;
    AMainRight,ARight,ATop,ABottom,AMainTop,AMainBottom,TryTop,tryBottom,TryRight : Integer;
    ACurrSize,AFirstShift : TSize;
    i : integer;
    s,ds,sysS : string;
    sysNumber : integer;
    IsUp,IsDown,LastUp,LastDown : boolean;
    ABitmap : TBitmap;
    SubSSize : TSize;
    NImage : integer;
begin
  if Transparence then self.Canvas.Brush.Style:=bsClear;
  AFont:=TFont.Create;
  AFont.Assign(self.Font);

  LastFont:=TFont.Create;
  LastFont.Assign(self.Font);
  self.canvas.Font.Assign(AFont);
  case FTextOrientation of
    iacbto_180_Degrees: RotateFont(self.canvas.Font,180);
    iacbto_90_Degrees : RotateFont(self.canvas.Font,90);
    iacbto_270_Degrees :RotateFont(self.canvas.Font,270);
  end;
  ABitmap:=TBitmap.Create;
  ABitmap.PixelFormat:=pf24Bit;
  ABitmap.Width:=100;
  ABitmap.Height:=24;

  ACurrSize.cx:=0;ACurrSize.cy:=0;
  AFirstShift.cx:=0;AFirstShift.cy:=0;
  for i:=0 to FStringsArray.Count-1 do//At All strings
    begin
      s:=FStringsArray[i];  isUp:=false;isDown:=false;LastUp:=false;LastDown:=false;
      AMainRight:=FPosArray[i].cx;AMainTop:=FPosArray[i].cy;AMainBottom:=FPosArray[i].cy;//for mainstring
      ARight:=FPosArray[i].cx;ATop:=FPosArray[i].cy;ABottom:=FPosArray[i].cy;//for all string...
      sysNumber:=-1;
      repeat
        ds:=GetBeforeFirstSys(s);
        if ds='' then
          begin
            sysNumber:=GetFirstSys(s);
            case sysNumber of
              0 :
                begin
                  AFont.Style:=AFont.Style+[fsBold];
                end;
              1 :
                begin
                  AFont.Style:=AFont.Style-[fsBold];
                end;  
              2 :
                begin
                  AFont.Style:=AFont.Style+[fsItalic];
                end;
              3 :
                begin
                  AFont.Style:=AFont.Style-[fsItalic];
                end;
              4 :
                begin
                  AFont.Style:=AFont.Style+[fsUnderline];
                end;
              5 :
                begin
                  AFont.Style:=AFont.Style-[fsUnderline];
                end;
              6 :
                begin
                  isUp:=true;LastUp:=true;
                  isdown:=false;
                end;
              7 :
                begin
                  LastDown:=false;isUp:=false;
                end;
              8 :
                begin
                  isDown:=true;LastDown:=True;
                  isUp:=false;
                end;
              9 :
                begin
                  LastUp:=false;isDown:=false;
                end;
              11 : //here we changing the size...
                begin
                  AFont.Size:=strtointdef(sysS,AFont.Size);
                end;
              13 : //here we inserting the bitmap...
                begin
                  NImage:=strtointdef(sysS,0);
                  if (FCurrImageList<>nil)and assigned(FCurrImageList) and(FCurrImageList.Count>NImage) and (NImage>=0) then
                    begin
                      case FTextOrientation of
                        iacbto_0_Degrees,iacbto_180_Degrees :
                          begin
                            SubSSize.cx:=FCurrImageList.Width;
                            SubSSize.cy:=FCurrImageList.Height;
                          end;
                        iacbto_90_Degrees,iacbto_270_Degrees :
                          begin
                            SubSSize.cx:=FCurrImageList.Height;
                            SubSSize.cy:=FCurrImageList.Width;
                          end;
                      end;
                      //-- as main text
                      AMainTop:=AMainTop-(SubSSize.cy-(AMainBottom-AmainTop)) div 2;
                      AMainBottom:=AMainTop+SubSSize.cy;

                      case FTextOrientation of
                        iacbto_0_Degrees : FCurrImageList.draw(Canvas,GetAllGap+ARight,GetAllGap+AMainTop,NImage,True);
                        iacbto_90_Degrees : FCurrImageList.draw(Canvas,GetAllGap+AMainTop,Height-GetAllGap-ARight-FCurrImageList.Height,NImage,True);
                        iacbto_180_Degrees: FCurrImageList.draw(Canvas,width-GetAllGap-ARight-FCurrImageList.Width,
                                                                 Height-GetAllGap-AMainTop-FCurrImageList.Height,NImage,True);
                        iacbto_270_Degrees :FCurrImageList.draw(Canvas,width-GetAllGap-AMainTop-FCurrImageList.Width,GetAllGap+ARight,NImage,True);
                      end;

                      if ATop>AMainTop then ATop:=AMainTop;
                      if ABottom<AMainBottom then ABottom:=AMainBottom;
                      ARight:=ARight+subSSize.cx;
                      AMainRight:=ARight;
                    end;
                end;
            end; //End of Case sysnumber
          end else
          begin
            case sysNumber of
              10,12 : //<size><inline>
                begin//that means no! action - so waiting for next
                  sysS:=ds;
                end;
              else
                begin
                  if (not isUp) and (not isDown) then
                    begin
                      LastUp:=false;LastDown:=false;
                      AMainRight:=ARight;
                    end;
                  ABitmap.Canvas.Font.Assign(AFont);
                  self.Canvas.Font.Assign(AFont);
                  case FTextOrientation of
                    iacbto_180_Degrees: RotateFont(self.canvas.Font,180);
                    iacbto_90_Degrees : RotateFont(self.canvas.Font,90);
                    iacbto_270_Degrees :RotateFont(self.canvas.Font,270);
                  end;
                  GetTextExtentPoint32(ABitmap.Canvas.Handle,pChar(ds),length(ds),SubSSize);
                  if isUp then
                    begin
                      if lastDown then
                        begin
                          TryTop:=AMainTop-(SubSSize.cy div 2);
                          TryBottom:=TryTop+SubSSize.cy;
                          TryRight:=AMainRight+subSSize.cx;
                          if ATop>TryTop then ATop:=TryTop;
                          if Abottom<TryBottom then ABottom:=TryBottom;
                          if ARight<TryRight then ARight:=TryRight;
                          case FTextOrientation of
                            iacbto_0_Degrees : self.canvas.TextOut(GetAllGap+AMainRight,GetAllGap+TryTop,ds);
                            iacbto_90_Degrees : self.canvas.TextOut(GetAllGap+TryTop,Height-GetAllGap-AMainRight,ds);
                            iacbto_180_Degrees: self.canvas.TextOut(width-GetAllGap-AMainRight,Height-GetAllGap-TryTop,ds);
                            iacbto_270_Degrees :self.canvas.TextOut(width-GetAllGap-TryTop,GetAllGap+AMainRight,ds);
                          end;
                          LastDown:=false;
                          AMainRight:=ARight;
                        end else
                        begin
                          TryTop:=AMainTop-(SubSSize.cy div 2);
                          TryBottom:=TryTop+SubSSize.cy;
                          ARight:=AMainRight+subSSize.cx;
                          if ATop>TryTop then ATop:=TryTop;
                          if Abottom<TryBottom then ABottom:=TryBottom;
                          case FTextOrientation of
                            iacbto_0_Degrees : self.canvas.TextOut(GetAllGap+AMainRight,GetAllGap+TryTop,ds);
                            iacbto_90_Degrees : self.canvas.TextOut(GetAllGap+TryTop,Height-GetAllGap-AMainRight,ds);
                            iacbto_180_Degrees: self.canvas.TextOut(width-GetAllGap-AMainRight,Height-GetAllGap-TryTop,ds);
                            iacbto_270_Degrees :self.canvas.TextOut(width-GetAllGap-TryTop,GetAllGap+AMainRight,ds);
                          end;
                        end;
                    end else
                    begin
                      if isDown then
                        begin
                          if LastUp then
                            begin
                              TryTop:=AMainBottom-(SubSSize.cy div 2);
                              TryBottom:=TryTop+SubSSize.cy;
                              TryRight:=AMainRight+subSSize.cx;
                              if ATop>TryTop then ATop:=TryTop;
                              if Abottom<TryBottom then ABottom:=TryBottom;
                              if ARight<TryRight then ARight:=TryRight;
                              case FTextOrientation of
                                iacbto_0_Degrees : self.canvas.TextOut(GetAllGap+AMainRight,GetAllGap+TryTop,ds);
                                iacbto_90_Degrees : self.canvas.TextOut(GetAllGap+TryTop,Height-GetAllGap-AMainRight,ds);
                                iacbto_180_Degrees: self.canvas.TextOut(width-GetAllGap-AMainRight,Height-GetAllGap-TryTop,ds);
                                iacbto_270_Degrees :self.canvas.TextOut(width-GetAllGap-TryTop,GetAllGap+AMainRight,ds);
                              end;
                              LastUp:=false;
                              AMainRight:=ARight;
                            end else
                            begin
                              TryTop:=AMainBottom-(SubSSize.cy div 2);
                              TryBottom:=TryTop+SubSSize.cy;
                              ARight:=AMainRight+subSSize.cx;
                              if ATop>TryTop then ATop:=TryTop;
                              if Abottom<TryBottom then ABottom:=TryBottom;

                              case FTextOrientation of
                                iacbto_0_Degrees : self.canvas.TextOut(GetAllGap+AMainRight,GetAllGap+TryTop,ds);
                                iacbto_90_Degrees : self.canvas.TextOut(GetAllGap+TryTop,Height-GetAllGap-AMainRight,ds);
                                iacbto_180_Degrees: self.canvas.TextOut(width-GetAllGap-AMainRight,Height-GetAllGap-TryTop,ds);
                                iacbto_270_Degrees :self.canvas.TextOut(width-GetAllGap-TryTop,GetAllGap+AMainRight,ds);
                              end;
                            end;
                        end else
                        begin

                          AMainTop:=AMainTop-(SubSSize.cy-(AMainBottom-AmainTop)) div 2;
                          AMainBottom:=AMainTop+SubSSize.cy;


                          case FTextOrientation of
                            iacbto_0_Degrees : self.canvas.TextOut(GetAllGap+ARight,GetAllGap+AMainTop,ds);
                            iacbto_90_Degrees : self.canvas.TextOut(GetAllGap+AMainTop,Height-GetAllGap-ARight,ds);
                            iacbto_180_Degrees: self.canvas.TextOut(width-GetAllGap-ARight,Height-GetAllGap-AMainTop,ds);
                            iacbto_270_Degrees :self.canvas.TextOut(width-GetAllGap-AMainTop,GetAllGap+ARight,ds);
                          end;

                          if ATop>AMainTop then ATop:=AMainTop;
                          if ABottom<AMainBottom then ABottom:=AMainBottom;
                          ARight:=ARight+subSSize.cx;
                          AMainRight:=ARight;
                        end;
                    end;
                end;
            end; //End of Case sysnumber
          end;
      until (ds='') and (sysNumber=-1);
  end;    
  ABitmap.free;
  AFont.Free;
  self.canvas.Font.Assign(LastFont);
  LastFont.free;
  if Transparence then self.Canvas.Brush.Style:=bsSolid;
end;

function IAChartBaseTextClass.GetAllGap: integer;
begin
  result:=FInnerGap+FOuterGap;
  case FBorderStyle of
    iacbbsNone : begin  end;
    iacbbsSimple : result:=result+1;
    iacbbsRising,iacbbsPressed : result:=result+2;
    iacbbsUp,iacbbsDown : result:=result+3;
  end;
end;

function IAChartBaseTextClass.GetBeforeFirstSys(var s: string): string;
var i,firstI,n : integer;
begin
  firstI:=0;
  for i:=low(SysWords) to high(SysWords) do
    begin
      n:=pos(SysWords[i],s);
      if firstI=0 then firstI:=n else
        begin
          if n>0 then
            begin
              if n<firstI then  firstI:=n;
            end
        end;
    end;
  if firstI>0 then
    begin
      result:=copy(s,1,firstI-1);
      s:=copy(s,firstI,length(s)-firstI+1);
    end else
    begin
      result:=s;
      s:='';
    end;
end;
function IAChartBaseTextClass.GetAntiColor(Value : Tcolor): Tcolor;
var
 R,G,B : Integer;
 V : Tcolor;

begin
  V:=ColortoRgb(Value);
  R:=(integer(V) shl 24) shr 24;
  G:=(integer(V) shl 16) shr 24;
  B:=(integer(V) shl 8)  shr 24;
  R:=255-R;G:=255-G;B:=255-B;
  Result:=RGB(R,G,B);
end;

function IAChartBaseTextClass.GetColor: TColor;
begin
  result:=inherited Color;
end;
function IAChartBaseTextClass.GetLColor(Value : Tcolor): Tcolor;
var
 R,G,B : Integer;
 V : Tcolor;

begin
  V:=ColortoRgb(Value);
  R:=(integer(V) shl 24) shr 24;
  G:=(integer(V) shl 16) shr 24;
  B:=(integer(V) shl 8)  shr 24;
  R:=R+32;G:=G+32;B:=B+32;
  if R>255 then R:=255;
  if G>255 then G:=255;
  if B>255 then B:=255;
  Result:=RGB(R,G,B);
end;
function IAChartBaseTextClass.GetDColor(Value : Tcolor): Tcolor;
var
 R,G,B : Integer;
  V : Tcolor;

begin
  V:=ColortoRgb(Value);
  R:=(integer(V) shl 24) shr 24;
  G:=(integer(V) shl 16) shr 24;
  B:=(integer(V) shl 8)  shr 24;
  R:=R-32;G:=G-32;B:=B-32;
  if R<0 then R:=0;
  if G<0 then G:=0;
  if B<0 then B:=0;
  Result:=RGB(R,G,B);
end;
function IAChartBaseTextClass.GetFirstSys(var s: string): integer;
var i,firstI,n : integer;
begin
  firstI:=0;result:=-1;
  for i:=low(SysWords) to high(SysWords) do
    begin
      n:=pos(SysWords[i],s);
      if firstI=0 then
        begin
          firstI:=n;result:=i;
        end else
        begin
          if n>0 then
            begin
              if n<firstI then begin firstI:=n; result:=i; end;
            end;
        end;
    end;
  if firstI>0 then
    begin
      s:=copy(s,firstI+length(SysWords[result]),length(s)-(firstI+length(SysWords[result]))+1);
    end else
    begin
      result:=-1;
    end;
end;

function IAChartBaseTextClass.GetFont: TFont;
begin
  result:= inherited Font;
end;

function IAChartBaseTextClass.GetText: TCaption;
begin
  result := inherited Text;
end;

procedure IAChartBaseTextClass.IfFontChanged(Sender: TObject);
begin
  DefineSizeShifting;
  invalidate;
end;

procedure IAChartBaseTextClass.Loaded;
begin
  inherited Loaded;
  DefineSizeShifting;
end;

procedure IAChartBaseTextClass.Paint;
var
  DColor,LColor : TColor;
begin
  if not Visible then exit;
  if not Transparence then
    begin
      canvas.Brush.Color:=color;
      canvas.FillRect(ClientRect);
    end;
  case FBorderStyle of
    iacbbsNone :
      begin

      end;
    iacbbsSimple :
      begin
        DColor:=GetAntiColor(ColoR);
        canvas.pen.color:=DColor;
        canvas.MoveTo(FOuterGap,FOuterGap);
        canvas.LineTo(Width-1-FOuterGap,FOuterGap);
        canvas.LineTo(Width-1-FOuterGap,Height-1-FOuterGap);
        canvas.LineTo(FOuterGap,Height-1-FOuterGap);
        canvas.LineTo(FOuterGap,FOuterGap);
      end;
    iacbbsRising :
      begin
        DColor:=GetAntiColor(ColoR);
        canvas.pen.color:=DColor;
        canvas.MoveTo(FOuterGap+1,FOuterGap+1);
        canvas.LineTo(Width-2-FOuterGap,FOuterGap+1);
        canvas.LineTo(Width-2-FOuterGap,Height-2-FOuterGap);
        canvas.LineTo(FOuterGap+1,Height-2-FOuterGap);
        canvas.LineTo(FOuterGap+1,FOuterGap+1);
        DColor:=GetDColor(ColoR);
        LColor:=GetLColor(Color);
        canvas.pen.color:=LColor;
        canvas.MoveTo(FOuterGap,FOuterGap);
        canvas.LineTo(Width-1-FOuterGap,FOuterGap);
        canvas.pen.color:=DColor;
        canvas.LineTo(Width-1-FOuterGap,Height-1-FOuterGap);
        canvas.LineTo(FOuterGap,Height-1-FOuterGap);
        canvas.pen.color:=LColor;
        canvas.LineTo(FOuterGap,FOuterGap);
      end;
    iacbbsPressed :
      begin
        DColor:=GetAntiColor(ColoR);
        canvas.pen.color:=DColor;
        canvas.MoveTo(FOuterGap+1,FOuterGap+1);
        canvas.LineTo(Width-2-FOuterGap,FOuterGap+1);
        canvas.LineTo(Width-2-FOuterGap,Height-2-FOuterGap);
        canvas.LineTo(FOuterGap+1,Height-2-FOuterGap);
        canvas.LineTo(FOuterGap+1,FOuterGap+1);
        DColor:=GetLColor(ColoR);
        LColor:=GetDColor(Color);
        canvas.pen.color:=LColor;
        canvas.MoveTo(FOuterGap,FOuterGap);
        canvas.LineTo(Width-1-FOuterGap,FOuterGap);
        canvas.pen.color:=DColor;
        canvas.LineTo(Width-1-FOuterGap,Height-1-FOuterGap);
        canvas.LineTo(FOuterGap,Height-1-FOuterGap);
        canvas.pen.color:=LColor;
        canvas.LineTo(FOuterGap,FOuterGap);
      end;
    iacbbsUp :
      begin
        DColor:=GetAntiColor(ColoR);
        canvas.pen.color:=DColor;
        canvas.MoveTo(FOuterGap+1,FOuterGap+1);
        canvas.LineTo(Width-2-FOuterGap,FOuterGap+1);
        canvas.LineTo(Width-2-FOuterGap,Height-2-FOuterGap);
        canvas.LineTo(FOuterGap+1,Height-2-FOuterGap);
        canvas.LineTo(FOuterGap+1,FOuterGap+1);
        DColor:=GetDColor(ColoR);
        LColor:=GetLColor(Color);
        canvas.pen.color:=LColor;
        canvas.MoveTo(FOuterGap,FOuterGap);
        canvas.LineTo(Width-1-FOuterGap,FOuterGap);
        canvas.pen.color:=DColor;
        canvas.LineTo(Width-1-FOuterGap,Height-1-FOuterGap);
        canvas.LineTo(FOuterGap,Height-1-FOuterGap);
        canvas.pen.color:=LColor;
        canvas.LineTo(FOuterGap,FOuterGap);
        DColor:=GetLColor(ColoR);
        LColor:=GetDColor(Color);
        canvas.pen.color:=LColor;
        canvas.MoveTo(FOuterGap+2,FOuterGap+2);
        canvas.LineTo(Width-1-FOuterGap-2,FOuterGap+2);
        canvas.pen.color:=DColor;
        canvas.LineTo(Width-1-FOuterGap-2,Height-1-FOuterGap-2);
        canvas.LineTo(FOuterGap+2,Height-1-FOuterGap-2);
        canvas.pen.color:=LColor;
        canvas.LineTo(FOuterGap+2,FOuterGap+2);
      end;
    iacbbsDown :
      begin
        DColor:=GetAntiColor(ColoR);
        canvas.pen.color:=DColor;
        canvas.MoveTo(FOuterGap+1,FOuterGap+1);
        canvas.LineTo(Width-2-FOuterGap,FOuterGap+1);
        canvas.LineTo(Width-2-FOuterGap,Height-2-FOuterGap);
        canvas.LineTo(FOuterGap+1,Height-2-FOuterGap);
        canvas.LineTo(FOuterGap+1,FOuterGap+1);
        DColor:=GetLColor(ColoR);
        LColor:=GetDColor(Color);
        canvas.pen.color:=LColor;
        canvas.MoveTo(FOuterGap,FOuterGap);
        canvas.LineTo(Width-1-FOuterGap,FOuterGap);
        canvas.pen.color:=DColor;
        canvas.LineTo(Width-1-FOuterGap,Height-1-FOuterGap);
        canvas.LineTo(FOuterGap,Height-1-FOuterGap);
        canvas.pen.color:=LColor;
        canvas.LineTo(FOuterGap,FOuterGap);
        DColor:=GetDColor(ColoR);
        LColor:=GetLColor(Color);
        canvas.pen.color:=LColor;
        canvas.MoveTo(FOuterGap+2,FOuterGap+2);
        canvas.LineTo(Width-1-FOuterGap-2,FOuterGap+2);
        canvas.pen.color:=DColor;
        canvas.LineTo(Width-1-FOuterGap-2,Height-1-FOuterGap-2);
        canvas.LineTo(FOuterGap+2,Height-1-FOuterGap-2);
        canvas.pen.color:=LColor;
        canvas.LineTo(FOuterGap+2,FOuterGap+2);
      end;
  end;
  DrawOnCanvasText;  
  inherited Paint;
end;

procedure IAChartBaseTextClass.RotateFont(AFont: TFont; ADeg: Integer);
var
  MyLFont : LogFONT;
begin
  GETOBJECT(AFont.Handle,sizeof(MyLFont),@MyLFont);
  MyLfont.lfEscapement:=ADeg*10;
  MyLFont.lfOrientation:=ADeg*10;
  AFont.Handle:=CREATEFONTINDIRECT(MyLFont);//As i think it will automatically delete previouse object...
  //as windows delete object...
end;

procedure IAChartBaseTextClass.SetAutoSize(Value: Boolean);
begin
  if FAutoSize <> Value then
    begin
      FAutoSize := Value;
      if Value then
        begin
          case FTextOrientation of
            iacbto_0_Degrees,iacbto_180_Degrees:
              begin
                width:=FSizeText.cx+2*GetAllGap;
                height:=FSizeText.cy+2*GetAllGap;
              end;
            iacbto_90_Degrees,iacbto_270_Degrees :
              begin
                width:=FSizeText.cy+2*GetAllGap;
                height:=FSizeText.cx+2*GetAllGap;
              end;
          end;
        end;
    end;
end;

procedure IAChartBaseTextClass.SetBorderStyle(Value: IAChartBaseBorderStyle);
begin
  if FBorderStyle<>Value then
    begin
      FBorderStyle := Value;
      DefineSizeShifting;
      invalidate;
    end;
end;

procedure IAChartBaseTextClass.SetColor(Value: TColor);
begin
  if inherited Color<>Value then
    begin
      inherited Color:=Value;
      invalidate;
    end;
end;

procedure IAChartBaseTextClass.SetFont(Value: TFont);
begin
  (inherited Font).Assign(Value);
  DefineSizeShifting;
  invalidate;
end;

procedure IAChartBaseTextClass.SetInnerGap(Value: integer);
begin
  if (FInnerGap<>Value ) and (FInnerGap>=0) then
    begin
      FInnerGap := Value;
      DefineSizeShifting;
      invalidate;
    end;
end;

procedure IAChartBaseTextClass.SetOuterGap(Value: integer);
begin
  if (FOuterGap<>Value ) and (FOuterGap>=0) then
    begin
      FOuterGap := Value;
      DefineSizeShifting;
      invalidate;
    end;
end;

procedure IAChartBaseTextClass.SetStringsArray(const Value: TStringList);
begin
  FStringsArray.Clear;
  FStringsArray.Assign(Value);
  Text:=FStringsArray.Text;
  DefineSizeShifting;
  invalidate;
end;

procedure IAChartBaseTextClass.SetText(Value: TCaption);
begin
  if inherited Text <> Value then
    begin
      inherited Text := Value;
      FStringsArray.Text:=inherited Text;
      DefineSizeShifting;
      invalidate;
    end;
end;

procedure IAChartBaseTextClass.SetTextOrientation(Value: IAChartBaseTextOrientation);
begin
  if   FTextOrientation<>Value then
    begin
      FTextOrientation := Value;
      if FAUtoSize then
        begin
          case FTextOrientation of
            iacbto_0_Degrees,iacbto_180_Degrees:
              begin
                width:=FSizeText.cx+2*GetAllGap;
                height:=FSizeText.cy+2*GetAllGap;
              end;
            iacbto_90_Degrees,iacbto_270_Degrees :
              begin
                width:=FSizeText.cy+2*GetAllGap;
                height:=FSizeText.cx+2*GetAllGap;
              end;
          end;
        end;
      DefineSizeShifting;  
      invalidate;  
    end;
end;



procedure IAChartBaseTextClass.SetCurrImageList(Value: TImageList);
begin
  FCurrImageList := Value;
  DefineSizeShifting;
  invalidate;
end;

function IAChartBaseTextClass.GetVisible: boolean;
begin
  result:=inherited Visible;
end;

procedure IAChartBaseTextClass.SetVisible(Value: boolean);
begin
  if (inherited Visible)<>Value then
    begin
      inherited Visible:=Value;
      invalidate;
    end;
end;

procedure IAChartBaseTextClass.SetAutoEditable(Value: Boolean);
begin
  FAutoEditable := Value;
end;

procedure IAChartBaseTextClass.WMLButtonDblClk(
  var Message: TWMLButtonDblClk);
begin
  inherited;
  if FAutoEditable then
    begin
      IAChartBaseTextEditWindow:=TIAChartBaseTextEditWindow.Create(self);
      IAChartBaseTextEditWindow.FCurrControl:=self;
      IAChartBaseTextEditWindow.ShowModal;
      case IAChartBaseTextEditWindow.ModalResult of
        mrNone,mrCancel :
          begin
            IAChartBaseTextEditWindow.release;
          end;
        mrOk   :
          begin
            IAChartBaseTextEditWindow.BackUpdate;
            IAChartBaseTextEditWindow.release;
          end;
        else
          begin
            IAChartBaseTextEditWindow.release;
          end;
      end;
    end;
end;

{ IAChartBaseTextClassData }

procedure IAChartBaseTextClassData.Assign(Source: TPersistent);
begin
  if Source is IAChartBaseTextClassData then
    begin
      Text:=(Source as IAChartBaseTextClassData).Text;
      Font:=(Source as IAChartBaseTextClassData).Font;
      Transparent:=(Source as IAChartBaseTextClassData).Transparent;
      Color:=(Source as IAChartBaseTextClassData).Color;
      Width:=(Source as IAChartBaseTextClassData).Width;
      Height:=(Source as IAChartBaseTextClassData).Height;
      Top:=(Source as IAChartBaseTextClassData).Top;
      Left:=(Source as IAChartBaseTextClassData).Left;
      AutoSize:=(Source as IAChartBaseTextClassData).AutoSize;
      TextOrientation:=(Source as IAChartBaseTextClassData).TextOrientation;
      BorderStyle:=(Source as IAChartBaseTextClassData).BorderStyle;
      InnerGap:=(Source as IAChartBaseTextClassData).InnerGap;
      OuterGap:=(Source as IAChartBaseTextClassData).OuterGap;
      CurrImageList:=(Source as IAChartBaseTextClassData).CurrImageList;
      Visible:=(Source as IAChartBaseTextClassData).Visible;
      AutoEditable:=(Source as IAChartBaseTextClassData).AutoEditable;
    end else
    begin
      inherited Assign(Source);
    end;
end;

constructor IAChartBaseTextClassData.Create(AOwner: TComponent);
begin
  inherited Create;
  FTextField := IAChartBaseTextClass.Create(AOwner as TWinControl);
  FTextField.Parent:=AOwner as TWinControl;
end;

destructor IAChartBaseTextClassData.Destroy;
begin
  FTextField.free;
  inherited Destroy;
end;

function IAChartBaseTextClassData.GetAutoEditable: Boolean;
begin
  result:=FTextField.AutoEditable;
end;

function IAChartBaseTextClassData.GetAutoSize: Boolean;
begin
  result:=FTextField.AutoSize;
end;

function IAChartBaseTextClassData.GetBorderStyle: IAChartBaseBorderStyle;
begin
  result:=FTextField.BorderStyle;
end;

function IAChartBaseTextClassData.GetColor: TColor;
begin
  result:=FTextField.color;
end;

function IAChartBaseTextClassData.GetCurrImageList: TImageList;
begin
  result:=FTextField.CurrImageList;
end;

function IAChartBaseTextClassData.GetFont: TFont;
begin
  result:=FTextField.Font;
end;

function IAChartBaseTextClassData.GetHeight: integer;
begin
  result:=FTextField.Height;
end;

function IAChartBaseTextClassData.GetInnerGap: integer;
begin
  result:=FTextField.InnerGap;
end;

function IAChartBaseTextClassData.GetLeft: Integer;
begin
  result:=FTextField.Left;
end;

function IAChartBaseTextClassData.GetOuterGap: integer;
begin
  result:=FTextField.OuterGap;
end;

function IAChartBaseTextClassData.GetStringsArray: TStringList;
begin
  result:=FTextField.StringsArray;
end;

function IAChartBaseTextClassData.GetText: TCaption;
begin
  result:=FTextField.Text;
end;

function IAChartBaseTextClassData.GetTextOrientation: IAChartBaseTextOrientation;
begin
  result:=FTextField.TextOrientation;
end;

function IAChartBaseTextClassData.GetTop: integer;
begin
  result:=FTextField.Top;
end;

function IAChartBaseTextClassData.GetTransparent: Boolean;
begin
  result:=FTextField.Transparence;
end;

function IAChartBaseTextClassData.GetVisible: Boolean;
begin
  result:=FTextField.Visible;
end;

function IAChartBaseTextClassData.GetWidth: integer;
begin
  result:=FTextField.width;
end;

function IAChartBaseTextClassData.IsYours(
  AObject: IAChartBaseTextClass): boolean;
begin
  if AObject=FTextField then result:=true else result:=false;
end;

procedure IAChartBaseTextClassData.SetAutoEditable(Value: Boolean);
begin
  FTextField.AutoEditable:=Value;
end;

procedure IAChartBaseTextClassData.SetAutoSize(Value: Boolean);
begin
  FTextField.AutoSize:=Value;
end;

procedure IAChartBaseTextClassData.SetBorderStyle(Value: IAChartBaseBorderStyle);
begin
  FTextField.BorderStyle:=Value;
end;

procedure IAChartBaseTextClassData.SetColor(Value: TColor);
begin
  FTextField.color:=Value;
end;

procedure IAChartBaseTextClassData.SetCurrImageList(Value: TImageList);
begin
  FTextField.CurrImageList:=Value;
end;

procedure IAChartBaseTextClassData.SetFont(Value: TFont);
begin
  FTextField.Font:=Value;
end;

procedure IAChartBaseTextClassData.SetHeight(Value: integer);
begin
  if not AutoSize then FTextField.Height:=Value;
end;

procedure IAChartBaseTextClassData.SetInnerGap(Value: integer);
begin
  FTextField.InnerGap:=Value;
end;

procedure IAChartBaseTextClassData.SetLeft(Value: Integer);
begin
  FTextField.Left:=Value;
end;

procedure IAChartBaseTextClassData.SetOuterGap(Value: integer);
begin
  FTextField.OuterGap:=Value;
end;

procedure IAChartBaseTextClassData.SetStringsArray(Value: TStringList);
begin
  FTextField.StringsArray:=Value;
end;

procedure IAChartBaseTextClassData.SetText(Value: TCaption);
begin
  FTextField.Text:=Value;
end;

procedure IAChartBaseTextClassData.SetTextOrientation(Value: IAChartBaseTextOrientation);
begin
  FTextField.TextOrientation:=Value;
end;

procedure IAChartBaseTextClassData.SetTop(Value: integer);
begin
  FTextField.Top:=Value;
end;

procedure IAChartBaseTextClassData.SetTransparent(Value: Boolean);
begin
  FTextField.Transparence:=Value;
end;

procedure IAChartBaseTextClassData.SetVisible(Value: Boolean);
begin
  FTextField.Visible:=Value;
end;

procedure IAChartBaseTextClassData.SetWidth(Value: integer);
begin
  if not AutoSize then FTextField.Width:=Value;
end;

end.
